/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish;

import junit.framework.TestCase;

import org.gvsig.publish.exceptions.InvalidRemoteResourceException;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.IViewInfo;
import org.gvsig.publish.infoproject.ViewInfo;
import org.gvsig.publish.serversmodel.Server;
import org.gvsig.publish.util.LocalResourcesTestUtilities;

import com.iver.cit.gvsig.project.documents.view.ProjectView;

public abstract class Test_publication extends TestCase {

	public LocalResourcesTestUtilities util = null;
	protected void setUp() throws Exception {
		super.setUp();		
		//gets the utilities
		try {
			util = new LocalResourcesTestUtilities();
		} catch (Exception e1) {		
			e1.printStackTrace();
		}
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	/**
	 * 
	 * @return server's type
	 */
	public abstract String getServerRegisterTag(); 
	/**
	 * testing shapefiles publication
	 * @throws InvalidRemoteResourceException 
	 */
	public void test_publish_shape() throws InvalidRemoteResourceException{
		Server server = null;
		//gets the server from the register
		server = PublishRegister.register().getServer(getServerRegisterTag());		
		this.assertNotNull("I can't get server from the register", server);
		
		//gets a gvSIG view with a shape from utilities
		//ProjectInfo inf = new ProjectInfo( l.getProject());
		//IViewInfo[] v =  inf.getViewsInfo();
		ProjectView v = util.getView_shape();
		IViewInfo iv = new ViewInfo(v,null);		
		server.addInfo(iv); 
		 
		//call publish()
		try {
			server.publish();
			System.err.println("Testing ... shapefiles");
		} catch (PublishException e) {
			System.err.println("Publish exception");
			e.printStackTrace();
		}
	}
	/**
	 * testing postgis publication
	 * @throws InvalidRemoteResourceException 
	 */
	public void test_publish_postgis() throws InvalidRemoteResourceException{
		Server server = null;
		//gets the server from the register
		server = PublishRegister.register().getServer(getServerRegisterTag());		
		this.assertNotNull("I can't get server from the register", server);
		
		ProjectView v = util.getView_postgis();
		IViewInfo iv = new ViewInfo(v,null);		
		server.addInfo(iv); 
		 
		//call publish()
		try {
			server.publish();
			System.err.println("Testing ... postgis");
		} catch (PublishException e) {
			System.err.println("Publish exception");
			e.printStackTrace();
		}
	}
	/**
	 * testing raster publication
	 * @throws InvalidRemoteResourceException 
	 */
	public void test_publish_raster() throws InvalidRemoteResourceException{
		Server server = null;
		//gets the server from the register
		server = PublishRegister.register().getServer(getServerRegisterTag());		
		this.assertNotNull("I can't get server from the register", server);
		
		ProjectView v = util.getView_raster();
		IViewInfo iv = new ViewInfo(v,null);		
		server.addInfo(iv); 
		 
		//call publish()
		try {
			server.publish();
			System.err.println("Testing ... raster");
		} catch (PublishException e) {
			System.err.println("Publish exception");
			e.printStackTrace();
		}
	}
	/**
	 * testing unique group layers publication
	 * @throws InvalidRemoteResourceException 
	 */
	public void test_publish_group() throws InvalidRemoteResourceException{
		Server server = null;
		//gets the server from the register
		server = PublishRegister.register().getServer(getServerRegisterTag());		
		this.assertNotNull("I can't get server from the register", server);
		
		ProjectView v = util.getView_group();
		IViewInfo iv = new ViewInfo(v,null);		
		server.addInfo(iv); 
		 
		//call publish()
		try {
			server.publish();
			System.err.println("Testing ... groups");
		} catch (PublishException e) {
			System.err.println("Publish exception");
			e.printStackTrace();
		}
	}
	/**
	 * testing points, polygons and lines legends.
	 * <p>
	 * testing multiple views
	 * <p>
	 * testing unique values and intervals legends
	 * @throws InvalidRemoteResourceException 
	 */
	public void test_publish_legends() throws InvalidRemoteResourceException{
		Server server = null;
		//gets the server from the register
		server = PublishRegister.register().getServer(getServerRegisterTag());		
		this.assertNotNull("I can't get server from the register", server);
		
		//unique values
		ProjectView v = util.getView_unique_values();
		IViewInfo iv = new ViewInfo(v,null);		
		server.addInfo(iv); 
		//intervals
		v = util.getView_intervals();
		iv = new ViewInfo(v,null);		
		server.addInfo(iv);
		//labeling
		v = util.getView_label();
		iv = new ViewInfo(v,null);		
		server.addInfo(iv);
		//points
		v = util.getView_point();
		iv = new ViewInfo(v,null);		
		server.addInfo(iv);
		//lines
		v = util.getView_line();
		iv = new ViewInfo(v,null);		
		server.addInfo(iv);
		//polygons
		v = util.getView_polygon();
		iv = new ViewInfo(v,null);		
		server.addInfo(iv);
		 
		//call publish()
		try {
			server.publish();
			System.err.println("Testing ... unique values, labeling, intervals, simple legends");
		} catch (PublishException e) {
			System.err.println("Publish exception");
			e.printStackTrace();
		}
	}
	/**
	 * TODO:We are testing the persistence
	 */
	public void test_persistence(){
		
	}
}
