/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject;

import junit.framework.TestCase;

import org.gvsig.publish.infoproject.datasources.DataSourceInfoFactory;
import org.gvsig.publish.infoproject.datasources.IDataBaseInfo;
import org.gvsig.publish.infoproject.datasources.IFileInfo;
import org.gvsig.publish.infoproject.datasources.IRasterDSInfo;
import org.gvsig.publish.util.LocalResourcesTestUtilities;

import com.iver.cit.gvsig.fmap.layers.FLayer;

public class DataSourceInfoTest extends TestCase {
	private LocalResourcesTestUtilities util = null;
	protected void setUp() throws Exception {
		super.setUp();
		util = new LocalResourcesTestUtilities();
		//creates the database postgis
		util.createDBPostGIS();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		//drops database
		//util.dropDBPostGIS();
	}
	/**
	 * Testing shapefiles
	 */
	public void test_shapefile(){
		System.err.println("testing shapefile info ...");
		FLayer layer=null;
		try {
			layer = util.getShapePolygon();
		} catch (Exception e) {		
			e.printStackTrace();
		}
		IDataSourceInfo datasource = DataSourceInfoFactory.getInstance(layer);
		//Testing the file info
		IFileInfo fileInfo = (IFileInfo) datasource;
		System.err.println("getFileName():" + fileInfo.getFileName());
		System.err.println("getAbsolutePath():" + fileInfo.getAbsolutePath());
		System.err.println("getExtension():" + fileInfo.getExtension());
		System.err.println("getFileNameWothoutExtension():" + fileInfo.getFilenameWithoutExtension());
		this.assertNotNull(fileInfo.getFile());
		System.err.println(datasource.getNativeBBox());
		System.err.println(datasource.getLatLonBBox());
		//Testing vectorial info
//		IVectorialDSInfo vecInfo = (IVectorialDSInfo) datasource;
//		System.err.println(vecInfo.getDataSourceType());
//		System.err.println(vecInfo.getGeomType());
	}
	/**
	 * Testing postgis table
	 */
	public void test_postgis(){
		System.err.println("testing postgis info ...");		
		FLayer layer=null;
		try {			
			layer = util.getView_postgis().getMapContext().getLayers().getLayer(0);		   	
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Object datasource = DataSourceInfoFactory.getInstance(layer);
		//Testing database info
		IDataBaseInfo dbInfo = (IDataBaseInfo) datasource;
		System.err.println(dbInfo.getHost());
		System.err.println(dbInfo.getPort());
		System.err.println(dbInfo.getUser());
		//Testing vectorial info
//		IVectorialDSInfo vecInfo = (IVectorialDSInfo) datasource;
//		System.err.println("type: " + vecInfo.getDataSourceType());
//		System.err.println("geom type: " +vecInfo.getGeomType());
	}
	/**
	 * Testing ECWs
	 * TODO: Change System.err.println() by asserts
	 */
	public void test_ecw(){
		System.err.println("testing rasterfile info ...");
		FLayer layer=null;
		try {
			layer = util.getLayerECW();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Object object = DataSourceInfoFactory.getInstance(layer);
		//Testing common datasource info
		IDataSourceInfo datasource = (IDataSourceInfo) object;
		print(datasource);
		//Testing the raster info
		IRasterDSInfo raster = (IRasterDSInfo) object;		
		print(raster);
		//Testing the file info
		IFileInfo fileInfo = (IFileInfo) object;
		print(fileInfo);		
	}
	public void test_tiff(){
		FLayer layer = util.getLayer_tiff();
		this.assertNotNull(layer);
		IDataSourceInfo datasource = DataSourceInfoFactory.getInstance(layer);
		print(datasource);
		//Testing the raster info
		IRasterDSInfo raster = (IRasterDSInfo) datasource;		
		print(raster);
		//Testing the file info
		IFileInfo file = (IFileInfo) datasource;
		print(file);
	}
	
	public void print(IDataSourceInfo datasource){
		System.err.println("getMaxX:" + datasource.getNativeBBox().getMaxX());
		System.err.println("getMaxY:" + datasource.getNativeBBox().getMaxY());
		System.err.println("getMinX:" + datasource.getNativeBBox().getMinX());
		System.err.println("getMinY:" + datasource.getNativeBBox().getMinY());
		
	}
	public void print(IRasterDSInfo raster){
		System.err.println("getTransparency():" + raster.getTransparency());		
		System.err.println("getNumBands" + raster.getNumBands());

	}
	public void print(IFileInfo file){
		System.err.println("getFileName:" + file.getFileName());
		System.err.println("getAbsolutePath:" + file.getAbsolutePath());				
	}
}
