/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject;

import junit.framework.TestCase;

import org.gvsig.publish.infoproject.legends.IIntervalLegendInfo;
import org.gvsig.publish.infoproject.legends.ISingleSymbolLegendInfo;
import org.gvsig.publish.infoproject.legends.IUniqueValuesLegendInfo;
import org.gvsig.publish.infoproject.legends.LegendInfoFactory;
import org.gvsig.publish.util.LocalResourcesTestUtilities;

import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.project.documents.view.ProjectView;

public class LegendInfoTest extends TestCase {
	LocalResourcesTestUtilities util = null;
	protected void setUp() throws Exception {
		super.setUp();
		util = new LocalResourcesTestUtilities();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	/**
	 * Test simple legend
	 */
	public void test_simple_legend(){
		ProjectView view = util.getView_shape();
		FLayers layers = view.getMapContext().getLayers();
		for ( int i = 0; i < layers.getLayersCount(); i++){
			FLayer layer = layers.getLayer(i);
			printLegend(LegendInfoFactory.getInstance(layer));
			
		}						
	}
	/**
	 * test unique values legend
	 */
	public void test_uniquevalues(){
		ProjectView view = util.getView_unique_values();
		FLayers layers = view.getMapContext().getLayers();
		for ( int i = 0; i < layers.getLayersCount(); i++){
			FLayer layer = layers.getLayer(i);
			printLegend(LegendInfoFactory.getInstance(layer));
			
		}						
		
	}
	/**
	 * test intervals legend
	 */
	public void test_intervals(){
		ProjectView view = util.getView_intervals();
		FLayers layers = view.getMapContext().getLayers();
		for ( int i = 0; i < layers.getLayersCount(); i++){
			FLayer layer = layers.getLayer(i);
			printLegend(LegendInfoFactory.getInstance(layer));
			
		}						
		
	}
	private void printLegend (Object legend){
		if (legend instanceof ISingleSymbolLegendInfo){
			printLegend((ISingleSymbolLegendInfo)legend);
		}
		if (legend instanceof IIntervalLegendInfo){
			printLegend((IIntervalLegendInfo)legend);
		}
		if (legend instanceof IUniqueValuesLegendInfo){
			printLegend((IUniqueValuesLegendInfo)legend);
		}
	}
	/**
	 * print simple legend parameters
	 * @param legend
	 */
	private void printLegend(ISingleSymbolLegendInfo legend){
		System.err.println("SLD:");
		System.err.println(legend.getSLDString());
	}
	/**
	 * print interval legend properties
	 * @param legend
	 */
	private void printLegend(IIntervalLegendInfo legend){
		System.err.println("getFileterItem(): " + legend.getFilterItem());
		System.err.println("getExpressions(): ");
		String[] expressions = legend.getExpressions();
		for (int i = 0; i< expressions.length; i++){
			System.err.println(expressions[i]);
		}
		System.err.println("getExpressions(): ");
		String[] intervals = legend.getIntervals();
		for (int i = 0; i< intervals.length; i++){
			System.err.println(intervals[i]);
		}
		System.err.println("SLD:");
		System.err.println(legend.getSLDString());
	}
	/**
	 * print unique values properties
	 * @param legend
	 */
	private void printLegend(IUniqueValuesLegendInfo legend){
		System.err.println("getFilterItem():" + legend.getFilterItem());
		System.err.println("getUniqueValues(): ");
		String[] values = legend.getUniqueValues();
		for (int i = 0; i< values.length; i++){
			System.err.println(values[i]);
		}
		System.err.println("SLD:");
		System.err.println(legend.getSLDString());
	}

}
