/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.serversmodel;

import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.exceptions.InvalidRemoteResourceException;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IProjectInfo;
import org.gvsig.publish.infoproject.IViewInfo;
import org.gvsig.publish.infoproject.ProjectInfoFactory;
import org.gvsig.publish.util.LocalResourcesTestUtilities;

import com.iver.utiles.XMLEntity;

public abstract class RemoteResourceTest extends TestCase {
	protected Server server = null;
	protected Service service = null;
	protected RemoteResource parent = null;	
	protected RemoteResource child = null;
	protected LocalResourcesTestUtilities util = null;
	
	public abstract String getRemoteResourceRegisterTag();
	public abstract Class getRemoteResourceClass();
	public abstract String getServiceRegisterTag();
	public abstract Class getServiceClass();
	public abstract String getServerRegisterTag();
	public abstract Class getServerClass();
	
	public RemoteResourceTest() throws InvalidRemoteResourceException {
		super();
		//create utilities
		try {
			util = new LocalResourcesTestUtilities();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//register server
		PublishRegister.register().addServer(getServerRegisterTag(), getServerClass());
		server = PublishRegister.register().getServer(getServerRegisterTag());
		try {
			server.setServerURL(new URL("http://server"));
		} catch (MalformedURLException e) {
			fail("I can't set the server URL");
		}

		assertNotNull(server);
		//register service
		PublishRegister.register().addService(getServerRegisterTag(),getServiceRegisterTag(), getServiceClass());
		service = PublishRegister.register().getService(server, getServiceRegisterTag());
		assertNotNull(service);
		server.addService(service);
		//register remote resource
		PublishRegister.register().addRemoteResource(getServerRegisterTag(), getServiceRegisterTag(), getRemoteResourceRegisterTag(), getRemoteResourceClass());
		parent = PublishRegister.register().getRemoteResource(service, getRemoteResourceRegisterTag());
		assertNotNull(parent);
		child = PublishRegister.register().getRemoteResource(service, getRemoteResourceRegisterTag());
		assertNotNull(child);
		
		//init parent
		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_shape());
		IViewInfo[] views = proj.getViewsInfo();
		ILayerInfo[] layers = views[0].getLayers();		
		parent.setLayerInfo(layers[0]);
		//I need to set the layerinfo before add the remote resource
		service.addRemoteResource(parent);
		
		//init child
		proj = ProjectInfoFactory.getProjectInfo(util.getProject_shape());
		views = proj.getViewsInfo();
		layers = views[0].getLayers();		
		child.setLayerInfo(layers[1]);
	}
	
//	public abstract RemoteResource getRemoteResource();
//	public abstract RemoteResource getChild();
	
//	public void testRemoteResource() {
//		fail("Not yet implemented");
//	}

	public void testGetId() {
		assertNotNull(parent.getId());
	}

	public void testGetRegisterTag() {
		assertNotNull(parent.getRegisterTag());
	}

	public void testGetService() {
		assertNotNull(parent.getService());
	}

	public void testGetLayerInfo() { 
		assertNotNull(parent.getLayerInfo());
	}

	public void testGetChildsCount() {
		int c = parent.getChildsCount();
		parent.addChild(child);
		assertEquals(c+1, parent.getChildsCount());
	}

	public void testGetChild() {		
		int c = parent.getChildsCount();
		parent.addChild(child);
		assertEquals(child, parent.getChild(c));
	}

	public void testAddChild() {
		int c = parent.getChildsCount();
		parent.addChild(child);
		assertEquals(c+1, parent.getChildsCount());
	}

	public void testRemove() {
		RemoteResource r = parent;
		Service s = parent.getService();
		int c = s.getRemoteResourcesCount();
		r.remove();
		assertEquals(c-1, s.getRemoteResourcesCount());
	}

	public void testPublish() throws PublishException {
		parent.publish();
	}

	public void testSetLayerInfo() throws Exception {
		LocalResourcesTestUtilities util = new LocalResourcesTestUtilities();
		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_shape());
		IViewInfo[] views = proj.getViewsInfo();
		ILayerInfo[] layers = views[0].getLayers();		
		parent.setLayerInfo(layers[0]);
		assertEquals(layers[0].getName(),parent.getLayerInfo().getName());
	}

	public void testGetXMLEntity() {
		assertNotNull(parent.getXMLEntity());
	}

	public void testSetXMLEntity() {
		XMLEntity xml = parent.getXMLEntity();
		parent.setXMLEntity(xml);
		//assertEquals(xml, parent.getXMLEntity());
	}

}
