/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.serversmodel;

import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IProjectInfo;
import org.gvsig.publish.infoproject.IViewInfo;
import org.gvsig.publish.infoproject.ProjectInfoFactory;
import org.gvsig.publish.util.LocalResourcesTestUtilities;

import com.iver.utiles.XMLEntity;
/**
 * Tests which any server must check.
 * Any service must be tested extending this class.
 * This class test methods that will be called the publish framework.   
 * 
 * @author jvhigon
 *
 */
public abstract class ServerTest extends TestCase {
	protected Server server = null;
	protected Service service = null;

	public abstract String getServiceRegisterTag();
	public abstract Class getServiceClass();
	public abstract String getServerRegisterTag();
	public abstract Class getServerClass();
	
	public ServerTest() {
		super();
		//register server
		PublishRegister.register().addServer(getServerRegisterTag(), getServerClass());
		server = PublishRegister.register().getServer(getServerRegisterTag());
		try {
			server.setServerURL(new URL("http://server"));
		} catch (MalformedURLException e) {
			fail("I can't set the server URL");
		}
		assertNotNull(server);
		//register service
		PublishRegister.register().addService(getServerRegisterTag(),getServiceRegisterTag(), getServiceClass());
		service = PublishRegister.register().getService(server, getServiceRegisterTag());
		assertNotNull(service);
	}
	public void testGetId() {
		assertNotNull(server.getId());
	}

	public void testGetRegisterTag(){
		assertNotNull(server.getRegisterTag());
	}

	public void testAddInfoIViewInfo() throws Exception {
		LocalResourcesTestUtilities util = new LocalResourcesTestUtilities();
		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_complete());
		IViewInfo[] views = proj.getViewsInfo();
		for (int i = 0; i < views.length; i++ ){
			server.addInfo(views[i]);
		}		
	}

	public void testAddInfoILayerInfo() throws Exception {
		LocalResourcesTestUtilities util = new LocalResourcesTestUtilities();
		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_complete());
		IViewInfo[] views = proj.getViewsInfo();
		for (int i = 0; i < views.length; i++ ){
			ILayerInfo[] layers = views[i].getLayers();
			for (int j = 0; j < layers.length; j++){
				server.addInfo(layers[j]);
			}
		}		
	}

	public void testGetServiceString() {
		server.addService(service);
		assertEquals(service,server.getService(service.getId()));
	}

	public void testAddService(){
		int c = server.getServicesCount();
		server.addService(service);		
		assertEquals(c + 1, server.getServicesCount());
		for (int i = 0; i < server.getServicesCount(); i ++){
			assertNotNull(server.getService(i));
		}		
	}

	public void testPublish() throws Exception {
		LocalResourcesTestUtilities util = new LocalResourcesTestUtilities();
		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_complete());
		IViewInfo[] views = proj.getViewsInfo();
		for (int i = 0; i < views.length; i++){
			server.addInfo(views[i]);;
		}		
		server.publish();
	}

//	public void testPostInitialize() {
//		fail("Not yet implemented");
//	}

	public void testGetServerURL() throws MalformedURLException {
		URL url = new URL("http://server.com/service");
		server.setServerURL(url);
		assertNotNull(server.getServerURL());
	}

	public void testSetServerURL() throws MalformedURLException {
		URL url = new URL("http://server.com/service");
		server.setServerURL(url);
	}

	public void testGetXMLEntity() throws MalformedURLException {
		URL url = new URL("http://server.com/service");
		server.setServerURL(url);
		assertNotNull(server.getXMLEntity());		
	}

	public void testSetXMLEntity() throws MalformedURLException {
		URL url = new URL("http://server.com/service");
		server.setServerURL(url);
		XMLEntity xml = server.getXMLEntity();
		server.setXMLEntity(xml);
	}

	public void testRemoveService() {
		int c = server.getServicesCount();		
		server.addService(service);
		server.removeService(service);
		assertEquals(c, server.getServicesCount());
	}

}
