/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.serversmodel;

import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.exceptions.InvalidRemoteResourceException;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IProjectInfo;
import org.gvsig.publish.infoproject.IViewInfo;
import org.gvsig.publish.infoproject.ProjectInfoFactory;
import org.gvsig.publish.infoproject.ViewInfo;
import org.gvsig.publish.util.LocalResourcesTestUtilities;

import com.iver.cit.gvsig.project.documents.view.ProjectView;
import com.iver.utiles.XMLEntity;
/**
 * Test which any service must check. 
 * Any service must be tested extending this class.
 * This class test methods that will be called the publish framework.  
 * 
 * @author jvhigon
 *
 */
public abstract class ServiceTest extends TestCase {

	protected Server server = null;
	protected Service service = null;
	protected RemoteResource remote = null;	
	protected LocalResourcesTestUtilities util = null;

	public abstract String getRemoteResourceRegisterTag();
	public abstract Class getRemoteResourceClass();
	public abstract String getServiceRegisterTag();
	public abstract Class getServiceClass();
	public abstract String getServerRegisterTag();
	public abstract Class getServerClass();

	public ServiceTest() {
		super();
		//create utilities
		try {
			util = new LocalResourcesTestUtilities();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//register server
		PublishRegister.register().addServer(getServerRegisterTag(), getServerClass());
		server = PublishRegister.register().getServer(getServerRegisterTag());
		assertNotNull(server);
		try {
			server.setServerURL(new URL("http://server"));
		} catch (MalformedURLException e) {
			fail("I can't set the server URL");
		}

		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_shape());		
		
		//register service
		PublishRegister.register().addService(getServerRegisterTag(),getServiceRegisterTag(), getServiceClass());
		service = PublishRegister.register().getService(server, getServiceRegisterTag());
		assertNotNull(service);
		server.addService(service);
		//register remote resource
		PublishRegister.register().addRemoteResource(getServerRegisterTag(), getServiceRegisterTag(), getRemoteResourceRegisterTag(), getRemoteResourceClass());
		remote = PublishRegister.register().getRemoteResource(service, getRemoteResourceRegisterTag());
		assertNotNull(remote);
		//init remote (I need to set the layerinfo before add the remote resource)
		
		IViewInfo[] views = proj.getViewsInfo();
		ILayerInfo[] layers = views[0].getLayers();		
		remote.setLayerInfo(layers[0]);
	}


	public void testGetRegisterTag() {
		assertNotNull(service.getRegisterTag());
	}

	public void testGetId() {
		assertNotNull(service.getId());
	}

	public void testGetFilter() {
		if (service.getFilter()==null){
			System.out.println("Service without filter");
		}
	}

	
	public void testGetServer() {
		assertNotNull(service.getServer());
	}

	public void testPublish() throws PublishException {
		service.publish();
	}

	public void testGetRemoteResourcesCount() {
		System.out.println(service.getRemoteResourcesCount());
	}

	public void testGetRemoteResource() throws InvalidRemoteResourceException {			
		service.addRemoteResource(remote);
		int c = service.getRemoteResourcesCount();
		assertEquals(remote, service.getRemoteResource(c-1));
	}

	public void testAddRemoteResource() throws InvalidRemoteResourceException {	
		int c = service.getRemoteResourcesCount();
		service.addRemoteResource(remote);
		assertEquals(c + 1, service.getRemoteResourcesCount());
	}

	public void testAddInfoIViewInfo() throws Exception {	
		ProjectView gvView = util.getView_shape();
		IViewInfo viewInfo = new ViewInfo(gvView,null);
		service.addInfo(viewInfo);
	}
	/**
	 * TODO: missing asserts
	 * @throws Exception
	 */
	public void testAddInfoILayerInfo() throws Exception {
		IProjectInfo proj = ProjectInfoFactory.getProjectInfo(util.getProject_shape());
		IViewInfo[] views = proj.getViewsInfo();
		ILayerInfo[] layers = views[0].getLayers();
		service.addInfo(layers[0]);				
	}

	public void testGetXMLEntity() {
		System.out.println(service.getXMLEntity().toString());
	}

	public void testSetXMLEntity() {
		XMLEntity xml = service.getXMLEntity();
		service.setXMLEntity(xml);
	}

	public void testRemoveRemoteResource() throws InvalidRemoteResourceException {

		int c = service.getRemoteResourcesCount();
		service.addRemoteResource(remote);
		service.removeRemoteResource(remote);
		assertEquals(c, service.getRemoteResourcesCount());
	}

	public void testRemove() {
		int c = server.getServicesCount();
		service.remove();
		assertEquals(c-1, server.getServicesCount());
	}

}
