/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/**
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 */
package org.gvsig.publish.util;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;

import org.cresques.cts.IProjection;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.FPolygon2D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.ConcreteMemoryDriver;
import com.iver.cit.gvsig.fmap.drivers.ConnectionFactory;
import com.iver.cit.gvsig.fmap.drivers.DBException;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.IConnection;
import com.iver.cit.gvsig.fmap.drivers.jdbc.postgis.PostGisDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.documents.exceptions.OpenException;
import com.iver.cit.gvsig.project.documents.view.ProjectView;
import com.iver.cit.gvsig.project.documents.view.ProjectViewFactory;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.xmlEntity.generate.XmlTag;

/**
 * This class represents a factory which can make gvSIG objects like a project,
 * a view, a layer, ... 
 * This class is useful to do tests.
 * If you want to use it, you must java the following libraries in the classpath:
 * jgdal, ecw, mrsid, geotiff, castor
 * And the projects: fmap, andami, iverutiles, appgvsig  
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class LocalResourcesTestUtilities {
	
	private String basePath = null;
	/**
	 * Default constructor
	 * <p>
	 *  it gets the resources from directory "resources" which is at the same level as the project extPublish
	 *  <p>
	 *  it configure gvSIG drivers path to ../fwAndami/....
	 */
	public LocalResourcesTestUtilities()throws Exception{
		File f = null;		
		f = new File("../extPublish/resources");
		if (!f.exists()) 
			throw new Exception("Can't find resources dir");		
		this.basePath = f.getAbsolutePath();
		System.out.println("RESOURCES DIRECTORY:" + basePath);
		//Load the drivers
		LayerFactory.setDriversPath("../_fwAndami/gvSIG/extensiones/com.iver.cit.gvsig/drivers");			
	}
	
	/**
	 * 
	 * @return Project with a shape
	 */
	public Project getProject_shape(){
		Project gvProj = ProjectFactory.createProject();
		gvProj.setOwner("test_project_owner");
		gvProj.setName("test_project_name");
		ProjectView view = getView_shape();
		gvProj.addDocument(view);
		return gvProj;
	}
	/**
	 * 
	 * @return Project with all the possible resources 
	 */
	public Project getProject_complete(){
		Project gvProj = ProjectFactory.createProject();
		gvProj.setOwner("test_project_owner");
		gvProj.setName("test_project_name");
		ProjectView view = getView_shape();
		gvProj.addDocument(view);
		view = getView_group();
		gvProj.addDocument(view);
		view = getView_raster();
		gvProj.addDocument(view);
		return gvProj;
	}
	/**
	 * 
	 * @return gvSIG view with 3 shapefiles, point, polygon and lines
	 */
	public ProjectView getView_shape(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/shapes.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with raster files, jpeg,tiff, ecw.
	 */
	public ProjectView getView_raster(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/raster.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with a postgis table (polygons)
	 */
	public ProjectView getView_postgis(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/postgis.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with two point shapes
	 */
	public ProjectView getView_point(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/point.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with a line shapes
	 */
	public ProjectView getView_line(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/line.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with a polygon shape
	 */
	public ProjectView getView_polygon(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/polygon.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with a polygon shape labeled
	 */
	public ProjectView getView_label(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/label.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with a group of layers (shapes) and a raster
	 */
	public ProjectView getView_group(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/group.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with several shapes, each one with a different SRS
	 */
	public ProjectView getView_srs(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/srs.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view with interval legend (shape of polygons)
	 */
	public ProjectView getView_intervals(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/intervals.xml");
		return view;
	}
	/**
	 * 
	 * @return gvSIG view unique values legend (shape line)
	 */
	public ProjectView getView_unique_values(){
		ProjectView view = createViewFromFile(this.basePath + "/xml/unique_values.xml");
		return view;
	}
	/**
	 * Creates a ProjectView from a XML file
	 */
	public ProjectView createViewFromFile(String fileName){
		ProjectView gvView =new ProjectView();
		gvView.setProjectDocumentFactory(new ProjectViewFactory());
		XMLEntity xmlEntity = createXMLEntityFromFile(fileName);
		System.err.println("Creating view from: " + fileName);
		try {
			gvView.setXMLEntity(xmlEntity);
		} catch (XMLException e) {
			e.printStackTrace();
		} catch (DriverException e) {
			e.printStackTrace();
		} catch (DriverIOException e) {
			e.printStackTrace();
		} catch (OpenException e) {
			e.printStackTrace();
		}
		return gvView;
	}
	/**
	 * Creates a XMLEntity from a file located in resources directory
	 * @param xml
	 * @return
	 */
	public XMLEntity createXMLEntityFromFile(String fileName){
		FileReader xml=null;
		try {
			//gvp = new FileReader(this.basePath + File.separator + fileName);
			xml = new FileReader(fileName);
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		Reader reader = new BufferedReader(xml);
		XmlTag tag=null;
		try {
			tag = (XmlTag) XmlTag.unmarshal(reader);
		} catch (MarshalException e) {
			e.printStackTrace();
		} catch (ValidationException e) {
			e.printStackTrace();
		}
		XMLEntity xmlEntity=new XMLEntity(tag);	
		return xmlEntity;
	}
	
	
	/**
	 * Creates an empty project
	 * @return gvSIG empty project (without documents)
	 */
	public Project getProject(){
		Project gvProj = ProjectFactory.createProject();
		gvProj.setOwner("test_project_owner");
		gvProj.setName("test_project_name");
		return gvProj;
	}
//	public Project getProject_gvp(){
//		FileReader gvp=null;
//		try {
//			gvp = new FileReader(this.basePath + File.separator +"project1.gvp");
//		} catch (FileNotFoundException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		}
//		Reader reader = new BufferedReader(gvp);
//		XmlTag tag=null;
//		try {
//			tag = (XmlTag) XmlTag.unmarshal(reader);
//		} catch (MarshalException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (ValidationException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		XMLEntity xml=new XMLEntity(tag);
//		Project p=null;
//		try {
//			p = Project.createFromXML(xml);
//		} catch (OpenException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		return p;
//	}
	
	
	/**
	 * Creates a basic view with one layer
	 * @deprecated
	 */
	public ProjectView getBasicView(){
		ProjectView gvView = new ProjectView();
		ViewPort viewPort = new ViewPort(CRSFactory.getCRS("EPSG:23031"));
		MapContext mapContext = new MapContext(viewPort);		
		mapContext.getLayers().addLayer(getLayerBasic());
		gvView.setMapContext(mapContext);
		gvView.setComment("test_view_comment");
		gvView.setCreationDate("07/07/07 13:30");
		gvView.setName("test_view_name");
		gvView.setOwner("test_view_owner");
		return gvView;
	}
	/**
	 * Creates a basic view with two layers
	 * @deprecated
	 */
	public ProjectView getBasic2layersView(){
		ProjectView gvView = new ProjectView();
		ViewPort viewPort = new ViewPort(CRSFactory.getCRS("EPSG:23031"));
		MapContext mapContext = new MapContext(viewPort);		
		mapContext.getLayers().addLayer(getLayerBasic());
		mapContext.getLayers().addLayer(getShapeLine());
		gvView.setMapContext(mapContext);
		gvView.setComment("test_view_comment");
		gvView.setCreationDate("07/07/07 13:30");
		gvView.setName("test_view_name");
		gvView.setOwner("test_view_owner");
		return gvView;
	}
	/**
	 * Creates a View with a simple layer and a nested layers
	 * @return
	 * @deprecated
	 */
	public ProjectView getGroupView(){
		ProjectView view = this.getBasicView();
		MapContext map = view.getMapContext();
		map.getLayers().addLayer(this.getGroupLayer(map));
		return view;
	}
	/**
	 * Creates a grouped layer
	 * @deprecated
	 */
	public FLayers getGroupLayer(MapContext mapCtxt){
		FLayers layers = null;
		layers = new FLayers(mapCtxt, null);
		layers.setName("test_grouped_layer");
		layers.addLayer(this.getShapeLine());
		layers.addLayer(this.getShapePoint());
		return layers;
	}
	/**
	 * Creates a vectorial layer with two geometries
	 * @deprecated 
	 */
	private FLayer getLayerBasic() {
		// DRIVER DEFINITION (SHAPE TYPE AND FIELDS)
		ConcreteMemoryDriver driver = new ConcreteMemoryDriver();
		driver.setShapeType(FShape.POLYGON);
		
		ArrayList arrayFields = new ArrayList();
		arrayFields.add("ID");
		arrayFields.add("NOMBRE");
		arrayFields.add("POBLACION");
		Value[] auxRow = new Value[3];
		
		driver.getTableModel().setColumnIdentifiers(arrayFields.toArray());
		
		// ATRIBUTES
		auxRow[0] = ValueFactory.createValue(0);
		auxRow[1] = ValueFactory.createValue("Valencia");
		auxRow[2] = ValueFactory.createValue(200000);
		
		// GEOMETRY 1 DEFINITION
		GeneralPathX gp1 = new GeneralPathX();
		gp1.moveTo(20,80);
		gp1.lineTo(80,60);
		gp1.lineTo(100, 140);
		
		FShape pol1 = new FPolygon2D(gp1);
		driver.addShape(pol1, auxRow);
		
		auxRow[0] = ValueFactory.createValue(1);
		auxRow[1] = ValueFactory.createValue("Madrid");
		auxRow[2] = ValueFactory.createValue(300000);
		
		// GEOMETRY 2 DEFINITION
		GeneralPathX gp2 = new GeneralPathX();
		gp2.moveTo(100,100);
		gp2.lineTo(110,100);
		gp2.lineTo(110,110);
		gp2.lineTo(100,110);
		gp2.lineTo(100,100);          
		
		FShape pol2 = new FPolygon2D(gp2);
		driver.addShape(pol2, auxRow);    
		
		// CREATE AND ADD LAYER
		FLayer lyr;
		String layerName = "basic_layer";
		
		IProjection proj = CRSFactory.getCRS("EPSG:23030");
		
		lyr = LayerFactory.createLayer(layerName,driver, proj);
		return lyr;				
	}
	/**
	 * creates a shapefile
	 * @deprecated
	 * @return
	 */
	public FLayer getShapePoint(){
		String name = "shp_point";
		String fileName = this.basePath + "/shp_point.shp";
		String driverName = "gvSIG shp driver";
		IProjection proj = CRSFactory.getCRS("EPSG:23030");
		FLayer lyr = null;
		try {
			lyr = LayerFactory.createLayer(name, driverName, new File(fileName), proj);			
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return (FLayer)lyr;
	}
	/**
	 * @deprecated
	 * @return
	 */
	public FLayer getShapeLine(){		
		String name = "shp_line";
		String fileName = this.basePath + "/shp_line.shp";
		String driverName = "gvSIG shp driver";
		IProjection proj = CRSFactory.getCRS("EPSG:23030");
		FLayer lyr = null;
		try {
			lyr = LayerFactory.createLayer(name, driverName, new File(fileName), proj);
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return (FLayer)lyr;
	}
	/**
	 * creates a shape of polygons   
	 * @throws Exception
	 * @Deprecated 
	 */
	public FLayer getShapePolygon(){		
		String name = "shp_polygon";
		String fileName = this.basePath + "/shp/shp_polygon.shp";
		String driverName = "gvSIG shp driver";
		IProjection proj = CRSFactory.getCRS("EPSG:23030");
		FLayer lyr = null;
		try {
			lyr = LayerFactory.createLayer(name, driverName, new File(fileName), proj);
			
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return (FLayer)lyr;
	}
	/**
	 * creates a test layer postGIS
	 * @deprecated
	 */
	public FLayer getLayerPostGIS(){
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		String dbURL = "jdbc:postgresql://localhost:5432/publish";
		String host="localhost";
		String user = "postgres";
		String pwd = "xxx";
		String layerName = "sql_polygon";
		String fidField = "gid";
		//String sFields = (String) args.get((String) "FIELDS");
		//String[] fields = sFields.split(",");
		String[] fields = {"gid", "area", "perimeter", "nucleos_", "nucleos_id", "cod_ent", "cod_ine", "nombre", "municipio", "cod_mun"};
		String geomField = "the_geom";
		String tableName = "sql_polygon";
		String whereClause = "";

		IConnection conn;
		try {
			conn = ConnectionFactory.createConnection(dbURL, user, pwd);
		} catch (DBException e) {
			e.printStackTrace();
			return null;
		}

		DBLayerDefinition lyrDef = new DBLayerDefinition();
		lyrDef.setName(layerName);
		lyrDef.setTableName(tableName);
		lyrDef.setWhereClause(whereClause);
		lyrDef.setFieldNames(fields);
		lyrDef.setFieldGeometry(geomField);
		lyrDef.setFieldID(fidField);
		//Why I must specify that once again?
		lyrDef.setUser(user);
		lyrDef.setPassword(pwd);
		lyrDef.setHost(host);
		lyrDef.setPort(5432);
		lyrDef.setShapeType(FShape.POLYGON);

		PostGisDriver pgd = new PostGisDriver();
		try {
			pgd.setData(conn, lyrDef);
		} catch (DBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return LayerFactory.createDBLayer(pgd, layerName, CRSFactory.getCRS("EPSG:23030"));

	}
	/**
	 * Creates a Raster Layer from a ECW
	 * @deprecated
	 */
	public FLayer getLayerECW(){
		//RasterDriver driv = new CmsRasterDriver();
		String name = "orto_ecw";
		String fileName = this.basePath + "/raster/orto_23030.ecw";
		String driverName = "gvSIG Image Driver";
		FLayer lyr=null;
		IProjection proj = CRSFactory.getCRS("EPSG:23030");
		//Driver driver = LayerFactory.getDM().getDriver("gvSIG Image Driver");		
		try {
			//lyr = LayerFactory.createLayer(name, (RasterDriver)driver, new File(fileName), proj);
			lyr = LayerFactory.createLayer(name, driverName, new File(fileName), proj);	
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}				
		return lyr;
	}
	
	/**
	 * Create DB PostGIS (publish)
	 * Preconditions:
	 * - exists commands createdb, psql
	 * - you don't need password to connect with the user postgres at localhost
	 * - exist a template postgis (with name template_postgis)
	 * TODO: pass all the arguments ant create a ProcessBuilder
	 *  
	 */
	public void createDBPostGIS (){
		//createDBPostGIS(String user, String password, String dbname, String host, String port)
		try {
			BufferedReader br = null;
			String line = null;
			String[] commands = null;
			Process child = null;
			 
		    // Creates the database
		   commands = new String[]{"createdb", "-U", "postgres", "-h","localhost", "-T", "template_postgis", "publish"};			    
		    child = Runtime.getRuntime().exec(commands);
		    //Show output
		    br = new BufferedReader(new InputStreamReader(child.getErrorStream()));
		    line = br.readLine();		    
		    while (line != null){		    	
		    	System.out.println(line);
		    	line = br.readLine();
		    }
		    // adds plpgsql support
//		    commands = new String[]{"createlang", "-U", "postgres","-h","localhost", "plpgsql", "publish"};		    
//		    child = Runtime.getRuntime().exec(commands);
//		    //Show output
//		    br = new BufferedReader(new InputStreamReader(child.getErrorStream()));
//		    line = br.readLine();		    
//		    while (line != null){		    	
//		    	System.out.println(line);
//		    	line = br.readLine();
//		    }
//		    
		    //load lwpostgis.sql
//		    commands = new String[]{"psql", "-U", "postgres","-h","localhost", "-d", "publish", "-f", this.basePath + "/postgis/lwpostgis.sql"};
//		    child = Runtime.getRuntime().exec(commands);
		    //Show output
//		    br = new BufferedReader(new InputStreamReader(child.getErrorStream()));
//		    line = br.readLine();		    
//		    while (line != null){		    	
//		    	System.out.println(line);
//		    	line = br.readLine();
//		    }
		    //load spatial_ref_sys.sql
//		    System.out.println("Loading spatial_ref_sys.sql");
//		    commands = new String[]{"psql", "-U", "postgres","-h","localhost", "-d", "publish", "-f", this.basePath + "/postgis/spatial_ref_sys.sql"};
//		    child = Runtime.getRuntime().exec(commands);
		  //Show output
//		    br = new BufferedReader(new InputStreamReader(child.getErrorStream()));
//		    line = br.readLine();		    
//		    while (line != null){		    	
//		    	System.out.println(line);
//		    	line = br.readLine();
//		    }
		    		    
		} catch (IOException e) {
			e.printStackTrace();
		}
		//Load a table
		this.loadTablePostGIS();
	}
	/**
	 * Drop DB PostGIS (publish)
	 */
	public void dropDBPostGIS(){
		try {
			BufferedReader br = null;
			String line = null;
			
		    // Execute a command with an argument that contains a space
		    String[] commands = new String[]{"dropdb", "-U", "postgres","-h","localhost", "publish"};		    
		    Process child = Runtime.getRuntime().exec(commands);	
		    //Show output
		    br = new BufferedReader(new InputStreamReader(child.getErrorStream()));
		    line = br.readLine();
		    while (line != null){
		    	System.out.println(line);
		    	line = br.readLine();
		    }
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	/**
	 * Load a table (resources/sql_polygon.sql) into the database
	 */
	private void loadTablePostGIS(){
		try {
			BufferedReader br = null;
			String line = null;			
		    String[] commands = new String[]{"psql", "-U", "postgres","-h", "localhost", "-d", "publish", "-f", this.basePath + "/postgis/sql_polygon.sql"};		    
		    Process child = Runtime.getRuntime().exec(commands);	
		    //Show output
		    br = new BufferedReader(new InputStreamReader(child.getErrorStream()));
		    line = br.readLine();
		    while (line != null){
		    	System.out.println(line);
		    	line = br.readLine();
		    }
		} catch (IOException e) {
			e.printStackTrace();
		}
	
	}
	public View getBasicGUIView() {
		View view = new View();
		ProjectView pView = getBasicView();
		view.initialize();
		view.setModel(pView);
		return view;
	}
	/*public Legend getSingleSymbolLegend() {
		SingleSymbolLegend leyenda = new SingleSymbolLegend();
		FSymbol symbol = new FSymbol();
		symbol.s
		leyenda.setDefaultSymbol(symbol);
		return leyenda;
	}
	*/
	public FLayer getLayer_tiff(){
		ProjectView view = getView_raster();
		FLayers layers = view.getMapContext().getLayers();
		return layers.getLayer(1);
	}
	
}
