/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish;

import java.util.Iterator;

import org.gvsig.publish.gui.properties.PublicationPropertiesController;
import org.gvsig.publish.gui.publish.PublishController;
import org.gvsig.publish.gui.selectServer.SelectServerController;
import org.gvsig.publish.infoproject.IProjectInfo;
import org.gvsig.publish.infoproject.ProjectInfoFactory;
import org.gvsig.publish.serversmodel.Publication;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.documents.ProjectDocument;
import com.iver.cit.gvsig.project.documents.exceptions.OpenException;
import com.iver.cit.gvsig.project.documents.exceptions.SaveException;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;
import com.iver.utiles.XMLEntity;

public class ProjectPublication extends ProjectDocument {
	/**
	 * 
	 */
	private static final long serialVersionUID = 5784873591226633893L;
	/*
	 * Dependences
	 * xxx
	 */
	private Publication publication = null;
	private PublishController publishCtrl = null;
	//private AddResourceController addResourceCtrl = null;
	private SelectServerController slcSrvCtrl = null;
	private PublicationPropertiesController ppc = null;
			
	/**
	 * Creates all the controllers and sets the relations.
	 * It's invoked when a new publication is created (new button)
	 * @see ProjectDocument#setProject(Project, int)
	 */
	public void setProject(Project project, int index) {
		super.setProject(project, index);							
	}
	/**
	 * 
	 * @return the publication of the document
	 */
	private Publication getPublication(){
		if (publication == null){
			publication = new Publication();
		}
		return publication;
	}
	
	/**
	 * @see ProjectDocument#afterAdd()
	 */
	public void afterAdd() {		
	
	}
	/**
	 * @see ProjectDocument#afterRemove()
	 */
	public void afterRemove() {
		// TODO Auto-generated method stub
		
	}
	/**
	 * This method is called when I open the document 
	 * @return
	 * @see {@linkProjectDocument#createWindow()}
	 */
	public IWindow createWindow() {
				 
		IProjectInfo projectInfo = ProjectInfoFactory.getProjectInfo(getProject());
		Iterator it= getPublication().setProjectInfo(projectInfo).iterator();
		String message="";
		while (it.hasNext()){
			message=message +" " +it.next().toString();
		}				
		//create controllers 
		publishCtrl = new PublishController();
		slcSrvCtrl = new SelectServerController();
		//initial message
		if (!message.equals("")){
			String aux = PluginServices.getText(this, "publish_layerinfo_not_available");
			message = aux + message;
			publishCtrl.setInitialMessage(message);
		}				
		
		//Add observers
		getPublication().addObserver(publishCtrl);
				
		//sets the publication in the controllers
		//addResourceCtrl.setPublication(publication);
		publishCtrl.setPublication(getPublication());
		slcSrvCtrl.setPublication(getPublication());
		
		

		//If the publication is new, I must select the server
		if (getPublication().getServer() == null){
			publishCtrl.showWindow();
			return slcSrvCtrl.getWindow();
		}		
		return publishCtrl.getWindow();
		//return null;
	}
	/**
	 * 
	 * @param root
	 * @param project
	 * @throws SaveException
	 * @see {@link ProjectDocument#exportToXML(XMLEntity, Project)}
	 */
	public void exportToXML(XMLEntity root, Project project)
			throws SaveException {
		XMLEntity tableRoot = project.getExportXMLTypeRootNode(root,ProjectTableFactory.registerName);
		tableRoot.addChild(this.getXMLEntity());
	}
	/**
	 * This method is invoked when the properties button is pressed
	 * 
	 * @return
	 * @see {@link ProjectDocument#getProperties()}
	 */
	public IWindow getProperties() {		
		if (ppc == null){
			ppc = new PublicationPropertiesController(getPublication());
		}
		return ppc.getWindow();
	}
	/**
	 * 
	 * @param root
	 * @param typeRoot
	 * @param elementIndex
	 * @param project
	 * @param removeDocumentsFromRoot
	 * @throws XMLException
	 * @throws DriverException
	 * @throws OpenException
	 * @see {@link ProjectDocument#importFromXML(XMLEntity, XMLEntity, int, Project, boolean)}
	 */  
	public void importFromXML(XMLEntity root, XMLEntity typeRoot,
			int elementIndex, Project project, boolean removeDocumentsFromRoot)
			throws XMLException, DriverException, OpenException {
		XMLEntity element = typeRoot.getChild(elementIndex);
		try {
			this.setXMLEntity(element);
		} catch (DriverIOException e) {			
			PublishLogger.getLog().error("ERROR ProjectPublication: importXML", e);
		}
		if (removeDocumentsFromRoot) {
			typeRoot.removeChild(elementIndex);
		}
		project.addDocument(this);
		
	}
	/**
	 * @see ProjectDocument#getXMLEntity()
	*/
	public XMLEntity getXMLEntity() throws SaveException {
		XMLEntity xml= super.getXMLEntity();
		xml.addChild(getPublication().getXMLEntity());		
		return xml;
	}
	/**
	 * @see ProjectDocument#getXMLEntity()
	 */
	public void setXMLEntity(XMLEntity xml) throws XMLException,
			DriverException, DriverIOException, OpenException {
		super.setXMLEntity(xml);
		getPublication().setXMLEntity(xml.getChild(0));	
	}
	
	
}
