/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import org.gvsig.publish.gui.publish.IPublishPluginController;
import org.gvsig.publish.serversmodel.RemoteResource;
import org.gvsig.publish.serversmodel.Server;
import org.gvsig.publish.serversmodel.Service;

import com.iver.utiles.extensionPoints.ExtensionPoint;
import com.iver.utiles.extensionPoints.ExtensionPoints;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;

/**
 * This class represents the register of the publish extension.
 * A publish plugin can register controllers, servers, services 
 * and remote resources.
 * <p>
 * Design pattern used: Singleton 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 * 
 * TODO: I must put the prefix into each string constant
 *
 */
public class PublishRegister {
//	static private String prefix = "publish";
	static private PublishRegister instance = null; 	
	private ExtensionPoints extensionPoints = null;
	
	/**
	 * 
	 * @return the publish's register
	 */
	static public PublishRegister register(){
		if(instance ==null){
			instance = new PublishRegister();
		}
		return instance;
	}
	/**
	 * Constructor:
	 * 
	 * Creates an Extension Point in order to register new servers
	 * 
	 * TODO: Is not necessary to create a extension point with "new"
	 * 
	 */
	protected PublishRegister(){		
		extensionPoints = ExtensionPointsSingleton.getInstance();	
	}
	/**
	 * Add a server to the register
	 * @param key
	 * @param clase
	 */
	public void addServer(String key, Class clase){		
		extensionPoints.add("publish_servers",key,clase);		
	}
	/**
	 * Get the names of the registered servers
	 * @return
	 */
	public ArrayList getServerNames(){
		Set set = null;
		Object o = extensionPoints.get("publish_servers");
		if (o == null){
			PublishLogger.getLog().error("ERROR PublishRegister: there isn't any server registered");
			return null;
		}else{
			ExtensionPoint ep = (ExtensionPoint) o;
			set = ep.keySet();
		}
		ArrayList list = new ArrayList();
		Iterator i = set.iterator();
		while (i.hasNext()){
			list.add(i.next());
		}
        return list;
	}
//	public Collection getServers(){
//		Collection collection = null;
//		Object o = extensionPoints.get(Server.PUBLISH_SERVERS);
//		if (o == null){
//			PublishLogger.getLog().error("ERROR PublishRegister: there isn't any server registered");
//		}else{
//			ExtensionPoint ep = (ExtensionPoint) o;
//			collection = ep.values();
//		}
//        return collection;
//	}
	/**
	 * Gets a server instance.
	 * The name of the extension point is "publish_servers".
	 * @param key
	 * @return
	 * 
	 */
	public Server getServer(String key){
		Server s = null;
		ExtensionPoint ep = null;
		ep = (ExtensionPoint) extensionPoints.get("publish_servers");
        try {
			 s = (Server) ep.create(key);
		} catch (InstantiationException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (IllegalAccessException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		}
		return s;
	}
	/**
	 * Adds service to the register
	 * @param server
	 * @param service
	 * @param clase
	 */
	public void addService(String server, String service, Class clase){				
		extensionPoints.add(server,service,clase);
	}
	/**
	 * 
	 * @param server string which identifies a server (server.REGISTER_TAG or server.getRegisterTag())
	 * @return names of the services registered by a server
	 * 
	 */
	public ArrayList getServicesNames(String server){
		ExtensionPoint ep = (ExtensionPoint) extensionPoints.get(server);
		Set set = ep.keySet();
		Iterator i = set.iterator();
		ArrayList list = new ArrayList();
		while (i.hasNext()){
			String aux = (String)i.next();			
			//aux = aux.split(server + "_")[1];			
			list.add(aux);
		}
        return list;
	}
	
//	public Collection getServices(Server server){
//		ExtensionPoint ep = (ExtensionPoint) extensionPoints.get(server.getType());
//		Collection collection = ep.values();		
//        return collection;
//	}
	
	/**
	 * Gets a service instance. The steps are: to register the server, 
	 * to register the service implemented by a server, finally, to get the servevice  
	 *  
	 * @param server string which identifies the server which registered the service
	 * @param service string which identifies the service to get (service.REGISTER_TAG or service.getRegisterTag()) 
	 * @return
	 */
	public Service getService(Server server, String service){
		Service s = null;
		ExtensionPoint ep = null;		
		ep = (ExtensionPoint) extensionPoints.get(server.getRegisterTag());
        try {
        	Object[] objects = new Object[1];
        	objects[0] = server;
			 s = (Service) ep.create(service, objects);
		} catch (InstantiationException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);			
		} catch (IllegalAccessException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (SecurityException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (IllegalArgumentException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (NoSuchMethodException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (InvocationTargetException e) {			
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		}
		return s;

	}
	/**
	 * Register a new remote resource. 
	 * @param service
	 * @param remoteResource
	 * @param clase
	 */
	public void addRemoteResource(String server, String service, String remoteResource, Class clase ){
		
		//extensionPoints = ExtensionPointsSingleton.getInstance();
		String aux = server + "#" + service;
		extensionPoints.add(aux,remoteResource,clase);
	}
	/**
	 * @param server 
	 * @param service
	 * @return names of the remote resources registered by a service. 
	 * Null if no one remote resource has been registered.
	 */
	public Set getRemoteResourcesNames(String server, String service){
		String aux= server + "#" + service;
		ExtensionPoint ep = (ExtensionPoint) extensionPoints.get(aux);
		Set set = null;
		if (ep !=null){
			set = ep.keySet();
		}		
		return set;
	}
	/**
	 * Gets a remote resource instance
	 * 
	 * @param service
	 * @param remoteReosurceName
	 * @return
	 */
	public RemoteResource getRemoteResource(Service service, String remoteReosurceName){
		RemoteResource r = null;
		String aux = service.getServer().getRegisterTag() + "#" + service.getRegisterTag();
		ExtensionPoint ep = (ExtensionPoint) extensionPoints.get(aux);
        try {			
        	Object[] objects = new Object[1];
        	objects[0] = service;
        	r = (RemoteResource) ep.create(remoteReosurceName,objects);
		} catch (InstantiationException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (IllegalAccessException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (SecurityException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (IllegalArgumentException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (NoSuchMethodException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (InvocationTargetException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		}
		return r;
	}
	/**
	 * Register a new gui controller
	 * @param key
	 * @param clase
	 */
	public void addController(String key, Class clase){		
		extensionPoints.add(IPublishPluginController.PUBLISH_PLUGIN_CONTROLLERS,key,clase);		
	}
	/**
	 * Get the names of the registered gui controllers
	 * @return Set
	 */
	public Set getControllersNames(){
		ExtensionPoint ep = (ExtensionPoint) extensionPoints.get(IPublishPluginController.PUBLISH_PLUGIN_CONTROLLERS);
        Set set = ep.keySet();
        return set;
	}
	/**
	 * 
	 * @param key
	 * @return
	 * TODO: This method should throw a PublishException
	 */
	public IPublishPluginController getController(String key){
		IPublishPluginController ctr = null;		
		ExtensionPoint ep = (ExtensionPoint) extensionPoints.get(IPublishPluginController.PUBLISH_PLUGIN_CONTROLLERS);
        try {
			 ctr = (IPublishPluginController) ep.create(key);
		} catch (InstantiationException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		} catch (IllegalAccessException e) {
			PublishLogger.getLog().error("ERROR PublishRegister", e);
		}
		return ctr;
	}

}
