/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.addResource;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.JOptionPane;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.exceptions.InvalidRemoteResourceException;
import org.gvsig.publish.infoproject.FilterProjectInfo;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.IProjectInfo;
import org.gvsig.publish.infoproject.IViewInfo;
import org.gvsig.publish.serversmodel.Publication;

import com.iver.andami.PluginServices;


public class AddResourceController implements ActionListener,TreeSelectionListener {
	/*
	 * Events
	 */
	public static final String ADDRESOURCE_EVENT_ACCEPT="accept_add_resource";
	public static final String ADDRESOURCE_EVENT_CANCEL="cancel_add_resource";
	/*
	 *Dependencies 
	 */
	private Publication publication = null;
	//private IProjectInfo projectInfo = null;
	private AddResourceWindow window = null;

	private HashSet resourcesToAdd = new HashSet();
	/**
	 * Constructor:
	 * <p>
	 * Creates the window and sets the listener itself.
	 */
	public AddResourceController(){
		window = new AddResourceWindow();	
		window.setListener(this);
	}
	/**
	 * Shows the window in order to add resources
	 * TODO: At the moment only gets the project information from the first service (it can be filtered)
	 * If we want multiservice support, we must have an "activeservice" in this class in order to show 
	 * the suitable projectinfo tree
	 */
	public void showWindow(){
		//window.setModel(publication.getProjectInfo());
		FilterProjectInfo filter = 	publication.getServer().getService(0).getFilter();
		IProjectInfo pi = publication.getProjectInfo();
		pi.setFilter(filter);
		window.setModel(pi);
		//show the window
		window.showWindow();
	}
	/**
	 * Get actions from the controls panel
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(AddResourceController.ADDRESOURCE_EVENT_CANCEL)){	
			resourcesToAdd.clear();
			window.closeWindow();
			return;
		}
		if (e.getActionCommand().equals(AddResourceController.ADDRESOURCE_EVENT_ACCEPT)){						
			Iterator i = resourcesToAdd.iterator();
			while (i.hasNext()){
				try {
					Object o = i.next();
					if (o instanceof IViewInfo){
						IViewInfo viewInfo = (IViewInfo)o;
						publication.getServer().addInfo(viewInfo);				
					}
					if (o instanceof ILayerInfo){
						ILayerInfo layerInfo = (ILayerInfo)o;	
						publication.getServer().addInfo(layerInfo);			
					}
				}catch (InvalidRemoteResourceException ex){
					JOptionPane.showMessageDialog(window,
							ex.getMessage(),
							PluginServices.getText(this,"publish_invalid_remote_resource"),
							JOptionPane.ERROR_MESSAGE);
				}
			}
			window.closeWindow();
		}
	}
	/**
	 * Tree selection event
	 */
	public void valueChanged(TreeSelectionEvent e) {
		TreePath[] paths = e.getPaths();
		for (int i = 0; i < paths.length; i++){				
			if (e.isAddedPath(paths[i])){
				resourcesToAdd.add(paths[i].getLastPathComponent());
			}else{
				resourcesToAdd.remove(paths[i].getLastPathComponent());
			}									
		}
	}

	/**
	 * Sets the publication 
	 */
	public void setPublication(Publication publi){
		if (publi == null){
			PublishLogger.getLog().error("ERROR AddresourceController : the publication can't be nulll");
		}else{
			publication = publi;
		}
	}
	/**
	 * Gets the controller's panel
	 */
	public AddResourcePanel getWindow(){
		return this.window;
	}
}
