/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.browser;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import org.gvsig.publish.gui.addResource.AddResourceController;
import org.gvsig.publish.gui.publish.IPublishPluginController;
import org.gvsig.publish.gui.publish.IPublishPluginPanel;
import org.gvsig.publish.serversmodel.Publication;
import org.gvsig.publish.serversmodel.RemoteResource;
import org.gvsig.publish.serversmodel.Server;
import org.gvsig.publish.serversmodel.Service;


public abstract class BrowserController implements IPublishPluginController,TreeSelectionListener, ActionListener{
	/*
	 *Events 
	 */
	public static final String ADD_RESOURCE_EVENT="add_resource";
	public static final String REMOVE_RESOURCE_EVENT="remove_resource";
	/*
	 * Dependences 
	 */
	protected Publication model_publication = null;
	private BrowserPanel browserPanel;		
	
	/*
	 * Last object selected
	 */
	Object source = null;
	/**
	 * Constructor
	 * 
	 * Creates its view and set the controller like itself
	 */
	public BrowserController(){
		//publicationPanel = new DefaultPublicationPanel();
		browserPanel = new BrowserPanel();
		browserPanel.setListener(this);
	}

	/**
	 * Each server must provide its own panel for servers
	 * @return
	 */
	public abstract IPublishPluginPanel getServerPanel(Server server);
	public abstract IPublishPluginPanel getServicePanel(Service service);
	public abstract IPublishPluginPanel getRemoteResourcePanel(RemoteResource remoteResource);

	public Object getModel() {		
		return browserPanel.getModel();		
	}

	public IPublishPluginPanel getPanel() {		
		return browserPanel;
	}
	/**
	 * At first, I put the model into the plugin controller
	 */
	public void setModel(Object entityModel) {
		//cast to publication
		model_publication = (Publication) entityModel;
		browserPanel.setModel(model_publication);	
		//If I set the model I'll show the server params		
		//publicationPanel.setParamsPanel(getServerPanel());
		browserPanel.changePropertiesPanel(getServerPanel(model_publication.getServer()));
	}
	public void valueChanged(TreeSelectionEvent e) {
		//Object source = null;	
		if (e.getNewLeadSelectionPath() == null){
			source = e.getOldLeadSelectionPath().getLastPathComponent();
		}else{
			source = e.getNewLeadSelectionPath().getLastPathComponent();
		}		
		//before change the panel, I must update the model
		browserPanel.updateModel();
		if (source instanceof Server){				
			Server current = (Server)source;			
			browserPanel.changePropertiesPanel(getServerPanel(current));
		}
		if (source instanceof Service){
			Service current = (Service) source;				
			//publicationPanel.setParamsPanel(getServicePanel(current.getType()));
			browserPanel.changePropertiesPanel(getServicePanel(current));
		}
		if (source instanceof RemoteResource){
			RemoteResource current = (RemoteResource) source;				
			//publicationPanel.setParamsPanel(getServicePanel(current.getType()));
			browserPanel.changePropertiesPanel(getRemoteResourcePanel(current));
		}
	}

	public void actionPerformed(ActionEvent e) {		
		if (e.getActionCommand().equals(BrowserController.ADD_RESOURCE_EVENT)){
			AddResourceController addResourceCtrl = new AddResourceController();
			addResourceCtrl.setPublication(model_publication);
			addResourceCtrl.showWindow();
		}
		if (e.getActionCommand().equals(BrowserController.REMOVE_RESOURCE_EVENT)){
//			if (source instanceof Service){
//				Service current = (Service) source;
//				current.remove();
//			}
			if (source instanceof RemoteResource){
				RemoteResource current = (RemoteResource) source;
				current.remove();
				browserPanel.setModel(model_publication);
			}
		}
		
	}

}
