/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.browser;

import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.EventListener;

import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.event.TreeSelectionListener;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.gui.publish.IPublishPluginPanel;
import org.gvsig.publish.serversmodel.Publication;

import com.iver.andami.PluginServices;

public class BrowserPanel extends JPanel implements IPublishPluginPanel, MouseListener {
    /**
	 * 
	 */
	private static final long serialVersionUID = 5245125820568692382L;
	/*
     * The last IpublishPanel selected 
     */
	IPublishPluginPanel lastPanel = null;
	
    /** Creates new form BrowserPanel */
    public BrowserPanel() {
        initComponents();
        initLabels();
    }
    private void initLabels(){
    	addItem.setText(PluginServices.getText(this,"publish_add"));
    	removeItem.setText(PluginServices.getText(this, "publish_remove"));
    	NoServersLabel.setText(PluginServices.getText(this,"publish_no_servers_available"));
    }
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jPopupMenu1 = new javax.swing.JPopupMenu();
        addItem = new javax.swing.JMenuItem();
        removeItem = new javax.swing.JMenuItem();
        jSplitPane1 = new javax.swing.JSplitPane();
        BrowserTreePanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        PublicationJTree = new javax.swing.JTree();
        BrowserPropertiesPanel = new javax.swing.JPanel();
        BrowserPropertiesNPanel = new javax.swing.JPanel();
        NoServersLabel = new javax.swing.JLabel();

        
        jPopupMenu1.add(addItem);

        
        jPopupMenu1.add(removeItem);

        setLayout(new java.awt.GridBagLayout());

        jSplitPane1.setResizeWeight(0.33);
        BrowserTreePanel.setLayout(new java.awt.GridBagLayout());

        jScrollPane1.setViewportView(PublicationJTree);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 2);
        BrowserTreePanel.add(jScrollPane1, gridBagConstraints);

        jSplitPane1.setLeftComponent(BrowserTreePanel);

        BrowserPropertiesPanel.setLayout(new java.awt.BorderLayout());

        BrowserPropertiesNPanel.setLayout(new java.awt.GridBagLayout());

        
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 2, 5, 5);
        BrowserPropertiesNPanel.add(NoServersLabel, gridBagConstraints);

        BrowserPropertiesPanel.add(BrowserPropertiesNPanel, java.awt.BorderLayout.NORTH);

        jSplitPane1.setRightComponent(BrowserPropertiesPanel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jSplitPane1, gridBagConstraints);

    }// </editor-fold>                        
    
    
    // Variables declaration - do not modify                     
    private javax.swing.JPanel BrowserPropertiesNPanel;
    private javax.swing.JPanel BrowserPropertiesPanel;
    private javax.swing.JPanel BrowserTreePanel;
    private javax.swing.JLabel NoServersLabel;
    private javax.swing.JTree PublicationJTree;
    private javax.swing.JMenuItem addItem;
    private javax.swing.JPopupMenu jPopupMenu1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JMenuItem removeItem;
    // End of variables declaration                   
                     
        
    private Publication publication;
    
	public Object getModel() {
		updateModel();
		return publication;
	}

	public void setListener(EventListener listener) {
		//Sets tree listener
		PublicationJTree.addTreeSelectionListener((TreeSelectionListener) listener);
		//sets listener for mouse event in order to show the popup menu
		PublicationJTree.addMouseListener(this);
		//add events for the menu items
		addItem.addActionListener((ActionListener)listener);
		addItem.setActionCommand(BrowserController.ADD_RESOURCE_EVENT);
		removeItem.addActionListener((ActionListener)listener);
		removeItem.setActionCommand(BrowserController.REMOVE_RESOURCE_EVENT);
	}

	public void setModel(Object entityModel) {
		// sets the publication into the tree
		publication = (Publication)entityModel;		
		//TODO: Maybe the publication can have a method like getTreeModel()
		PublicationJTree.setModel(new ServersTreeModel(publication));		
	}

	public void updateModel() {
		//Before change the properties panel, I must update the model in the last selection
		if(lastPanel != null){
			lastPanel.getModel();
		}	
		//Repaint the container
		BrowserPropertiesPanel.doLayout();		
		
	}
	
	public void changePropertiesPanel(IPublishPluginPanel newPanel){				
		//Cast to JPanel
		JPanel panel = null;
		try{
			panel = (JPanel) newPanel;
		}catch(ClassCastException e){
			PublishLogger.getLog().error("ERROR DefaultPublicationPanel",e);
		}
		//remove old panel
		int i = jSplitPane1.getDividerLocation();
		BrowserPropertiesPanel.removeAll();
		BrowserPropertiesPanel.add(panel, java.awt.BorderLayout.NORTH);		
		jSplitPane1.remove(jSplitPane1.getRightComponent());
		jSplitPane1.add(BrowserPropertiesPanel,JSplitPane.RIGHT);		
		jSplitPane1.setDividerLocation(i);
		
		//Repaint the container
		BrowserPropertiesPanel.validate();		
		
		//set the lastPanel
		lastPanel = newPanel;		
	}


	public void mousePressed(MouseEvent e) {
		if ( e.isPopupTrigger() ) 
            jPopupMenu1.show( e.getComponent(),
                            e.getX(), e.getY() );
		
	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

    
}
