/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.properties;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JFileChooser;

import org.gvsig.publish.gui.common.PublishFileChooser;
import org.gvsig.publish.serversmodel.Publication;

import com.iver.andami.ui.mdiManager.IWindow;

/**
 * Controller for the publication properties GUI
 * Responsibilities: <br>
 * - creates GUI <br>
 * - process actions from the GUI and updates the model
 * 
 * @author jvhigon
 *
 */
public class PublicationPropertiesController implements ActionListener {
	//Events
	private final String ACCEPT_EVENT="accept";
	private final String CANCEL_EVENT="cancel";
	private final String FILE_EVENT="file"; 
	//Associations 
	private PublicationPropertiesWindow window = null;
	private Publication publication = null;
	private PublishFileChooser fileChooser = null;
	/**
	 * Constructor
	 */
	public PublicationPropertiesController(Publication pub) {
		//sets model
		this.publication = pub;
		
		//creates GUI
		window = new PublicationPropertiesWindow();
		fileChooser = new PublishFileChooser(new File(System.getProperty("user.home")));
		
		//set listeners
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		window.acceptButton.addActionListener(this);
		window.acceptButton.setActionCommand(ACCEPT_EVENT);
		window.cancelButton.addActionListener(this);
		window.cancelButton.setActionCommand(CANCEL_EVENT);
		window.fileButton.addActionListener(this);
		window.fileButton.setActionCommand(FILE_EVENT);
	}
	
	/**
	 * Query the model and initialize the GUI
	 * 
	 * @return GUI
	 */
	public IWindow getWindow(){
		window.localmpText.setText(publication.getLocalMountPoint());
		window.remotempText.setText(publication.getRemoteMountPoint());
		window.fileaccessCombo.setSelectedIndex(publication.getAccessFileMethod());
		return this.window;
	}
	
	/**
	 * GUI commands process for the actionPerformed event
	 */
	public void actionPerformed(ActionEvent arg0) {		
		
		if (arg0.getActionCommand().equals(ACCEPT_EVENT)){
			publication.setAccessFileMethod(window.fileaccessCombo.getSelectedIndex());
			publication.setLocalMountPoint(window.localmpText.getText());
			publication.setRemoteMountPoint(window.remotempText.getText());
			window.closeWindow();
		}
		if (arg0.getActionCommand().equals(CANCEL_EVENT)){			
			window.closeWindow();
		}
		if (arg0.getActionCommand().equals(FILE_EVENT)){			
			int returnVal = fileChooser.showSaveDialog(window);
			if (returnVal == JFileChooser.APPROVE_OPTION){
				File path = fileChooser.getSelectedFile();
				window.localmpText.setText(path.getAbsolutePath());										
			}
		}
	}

}
