/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.publish;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.serversmodel.Publication;

import com.iver.andami.PluginServices;

/**
 * This class represents the controller for the publish use case
 * <p>
 * Responsabilities:
 * <p>
 * To create the publish window
 * <p>
 * To get the actions from the window
 *   
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class PublishController implements ActionListener, Observer {
	//events
	public static final String PUBLISH_EVENT_ACCEPT="accept_publish";
	public static final String PUBLISH_EVENT_CANCEL="cancel_publish";
	//associations
	private PublishWindow window = null;
	private Publication publication = null;
	private IPublishPluginController pluginCtrl = null;
	private String initialMessage=null;
	
	
	/**
	 * Constructor
	 */
	public PublishController(){
			window = new PublishWindow();
			window.setListener(this);
	}
	/**
	 * 
	 * @return publication active
	 */
	public Publication getPublication(){
		return publication;
	}
	/**
	 * The publication class needs a IWindow
	 * @return IWindow 
	 */
	public PublishWindow getWindow(){
		if (getInitialMessage() != null){
			JOptionPane.showMessageDialog(window,				
				getInitialMessage(),
				PluginServices.getText(this,"publish_publication_has_changed"),
				JOptionPane.WARNING_MESSAGE);
		}
		updateWindow();
		return window;
	}
	/**
	 * Shown the controller's window.
	 * Preconditions: first you must call setPublication
	 */
	public void showWindow(){		
		window.showWindow();
	}	
	/**
	 * Sets the controller's publication
	 * 
	 * @param publication
	 */
	public void setPublication(Publication publication){
		//if there isn't publications it creates one
		if (publication==null){
			//TODO I need to throw an exception
			//PublishLogger.getLog().error("ERROR: PublishController, the active publication is null ", new PublishException());
			PublishLogger.getLog().error("ERROR: PublishController, the active publication is null");
		}else{	
			this.publication = publication;
		}		
	}
	
	/**
	 * Actions from publish window 
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(PublishController.PUBLISH_EVENT_CANCEL)){			
			window.closeWindow();
		}
		if (e.getActionCommand().equals(PublishController.PUBLISH_EVENT_ACCEPT)){			
			try {
				//TODO: I don't know if this is the best method
				//I'm forcing to do a GetModel()
				//Why I can't use the plugincontroller instead?				
				//window.getActivePublication();
				pluginCtrl.getModel();
				String aux = publication.getServer().publish();
				JOptionPane.showMessageDialog(window,
						PluginServices.getText(this,aux),
						PluginServices.getText(this,"publish_correct_publication"),						
						JOptionPane.INFORMATION_MESSAGE);										
			} catch (PublishException e1) {
				PublishLogger.getLog().error("ERROR PublishController: error_publishing");
				//show a dialog
				JOptionPane.showMessageDialog(window,						
						PluginServices.getText(this, e1.getMessage()),
						PluginServices.getText(this,"publish_error_publishing"),						
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			window.closeWindow();
		}		
	}
	/**
	 * @see java.util.Observer#update(Observable, Object)
	 */
	public void update(Observable o, Object arg) {
		if (o instanceof Publication){
			setPublication((Publication)o);			
			updateWindow();
		}else{
			PublishLogger.getLog().error("ERROR PublishController: The observable object must be a publication" );
		}
	}
	private void updateWindow() {
		if (publication ==null){
			PublishLogger.getLog().error("ERROR " + getClass().getName() + " I can't update the window, publication is null!!");
			return;
		}
		//get active publication
		if (publication.getServer()==null){
			return;
		}
		String key = publication.getServer().getRegisterTag();
		//get controller		
		pluginCtrl = (IPublishPluginController) PublishRegister.register().getController(key);
		if (pluginCtrl == null){
			PublishLogger.getLog().error("ERROR " + getClass().getName() + "The gui controller has not been loaded!!");
			window.closeWindow();
			return;
		}
		pluginCtrl.setModel(publication);
		//TODO: check casting
		IPublishPluginPanel publishPanel = pluginCtrl.getPanel();
		JPanel panel = null;
		try{
			panel = (JPanel) publishPanel;
		}catch(ClassCastException e){
			PublishLogger.getLog().error("ERROR Publish Controller",e);
		}	
		window.setPublicationPanel(panel);
	}
	/**
	 * @return the initialMessage
	 */
	public String getInitialMessage() {
		return initialMessage;
	}
	/**
	 * @param initialMessage the initialMessage to set
	 */
	public void setInitialMessage(String initialMessage) {
		this.initialMessage = initialMessage;
	}

}
