/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.selectServer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JOptionPane;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.PublishRegister;
import org.gvsig.publish.serversmodel.Publication;
import org.gvsig.publish.serversmodel.Server;
import org.gvsig.publish.serversmodel.Service;

import com.iver.andami.ui.mdiManager.IWindow;

/**
 * This class represents the controller for the "add server" use case.
 * It adds a server into a publication
 * <p>
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class SelectServerController implements ActionListener {
	/**
	 * Events
	 */
	public static final String NEWSERVER_EVENT_ACCEPT="accept_new_server";
	public static final String NEWSERVER_EVENT_CANCEL="cancel_new_server";
	public static final String COMBOSERVERURL_EVENT_CHANGE = "change_combo_serverurl";
	public static final String COMBOSERVER_EVENT_CHANGE = "change_combo_server";
	/**
	 * dependences
	 */
	//PublishController publishController = null;
	Publication publication = null;
	
	private SelectServerWindow window = null;
	/**
	 * Constructor:
	 * <p>
	 * Creates the window and sets the listener itself.
	 */
	public SelectServerController(){
		window = new SelectServerWindow();	
		window.setListener(this);		
	}
	/**
	 * Sets the publication 
	 * @param publi
	 */
	public void setPublication(Publication publi){
		if (publi == null){
			PublishLogger.getLog().error("ERROR AddresourceController : the publication can't be nulll");
		}else{
			publication = publi;
		}
	}
	/**
	 * Shows the window
	 */
	public void showWindow(){
		window.showWindow();
	}
	/**
	 * Get actions from the window
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(SelectServerController.NEWSERVER_EVENT_CANCEL)){			
			window.closeWindow();
			return;
		}
		if (e.getActionCommand().equals(SelectServerController.NEWSERVER_EVENT_ACCEPT)){
			String serverName = window.getSelectedServer();
			String serviceName = window.getSelectedService();
			String serverUrl = window.getSelectedURL();
			
			//Saves the selected server 
			window.saveCurrentServer();
			
			//Creates the new Server and sets url
			Server server = PublishRegister.register().getServer(serverName);
			URL url;
			try {
				url = new URL (serverUrl);
			} catch (MalformedURLException e1) {
				JOptionPane.showMessageDialog(window,"malformed_url","ERROR",JOptionPane.ERROR_MESSAGE);
				return;
			}
			server.setServerURL(url);
			
			//Creates the new Service
			Service service = PublishRegister.register().getService(server, serviceName);
			
			//Adds the service into the server
			server.addService(service);
			
			//adds the new server into the publication
			//then publication state changes and notify observers
			publication.setServer(server);
			
			//closes the add server window and shows the publication windows
			window.closeWindow();
			//I've changed byt the observer/observable pattern
			//publishController.showWindow();
		}else if(e.getActionCommand().equals(SelectServerController.COMBOSERVERURL_EVENT_CHANGE)){
			window.updateCombos();
			return;
		}
		if (e.getActionCommand().equals(SelectServerController.COMBOSERVER_EVENT_CHANGE)){			
			window.updateServiceCombo();
			return;
		}
	}
	/**
	 * Maybe is better to return the panel and do the cast in the other side. Then I won't have
	 * the dependency with gvSIG (IWindow) into the controller 
	 * @return
	 */
	public IWindow getWindow() {		
		return window;
	}
	
}
