/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import org.gvsig.publish.gui.addResource.AddResourceController;
import org.gvsig.publish.gui.publish.IPublishPluginController;
import org.gvsig.publish.gui.publish.IPublishPluginPanel;
import org.gvsig.publish.gui.remove.RemoveResourceController;
import org.gvsig.publish.serversmodel.Publication;
import org.gvsig.publish.serversmodel.RemoteResource;
import org.gvsig.publish.serversmodel.Server;
import org.gvsig.publish.serversmodel.Service;
/**
 * GUI controller for a publication wizard.  
 * 
 * @author jvhigon
 *
 */
public abstract class PublishWizardController implements IPublishPluginController, ActionListener, TreeSelectionListener, ChangeListener {
	//Associations
	private RemoveResourceController removeCtrl = null;
	private PublishWizardPanel panel;
	private Publication publication;
	private Server currentServer;
	private Service currentService;
	private RemoteResource currentRemoteResource;
	private IPublishPluginPanel serverBasicPane;
	private IPublishPluginPanel serverAdvancedPane;
	private IPublishPluginPanel serviceBasicPane;
	private IPublishPluginPanel serviceAdvancedPane;
	private IPublishPluginPanel remoteBasicPane;
	private IPublishPluginPanel remoteAdvancedPane;
	//Events
	public static final String ADD_RESOURCE_EVENT="add_resource_event";
	
	//Constructor
	/**
	 * Default constructor. It creates the panel and set its listener as "this".
	 */
	public PublishWizardController() {		
		this.panel = new PublishWizardPanel();
		this.panel.setListener(this);						
		//remove 
		removeCtrl = new RemoveResourceController(panel.removeButton, panel.remoteTree);		
	}
	//Abstract methods
	/**
	 * Any subclass must have a panel for server parameters
	 * @param server
	 * @return
	 */
	public abstract IPublishPluginPanel getServerPanel(Server server);
	/**
	 * Any subclass must have a panel for advanced server parameters
	 * @param server model I want to show 
	 * @return
	 */
	public abstract IPublishPluginPanel getServerAdvancedPanel(Server server);
	/**
	 * Any subclass must have a panel for service parameters
	 * @param service model I want to show
	 * @return
	 */
	public abstract IPublishPluginPanel getServicePanel(Service service);
	/**
	 * Any subclass must have a panel for advanced service parameters
	 * @param service model I want to show
	 * @return
	 */
	public abstract IPublishPluginPanel getServiceAdvancedPanel(Service service);
	/**
	 * Any subclass must have a panel for remote resources panel
	 * @param remoteResource model I want to show
	 * @return
	 */
	public abstract IPublishPluginPanel getRemoteResourcePanel(RemoteResource remoteResource);
	/**
	 * Any subclass must have a panel for advanced remote resources panel
	 * @param remoteResource model I want to show
	 * @return
	 */
	public abstract IPublishPluginPanel getRemoteResourceAdvancedPanel(RemoteResource remoteResource);
	
	//Methods
	/**
	 * Gets the publication from the GUI. The GUI will have to update the model first.
	 */
	public Object getModel() {			
		serverBasicPane.getModel();		
		if (serverAdvancedPane != null){
			serverAdvancedPane.getModel();
		}
		serviceBasicPane.getModel();
		if (serviceAdvancedPane !=null){
			serviceAdvancedPane.getModel();
		}
		if (remoteBasicPane != null)
				remoteBasicPane.getModel();
		if (remoteAdvancedPane != null) 
				remoteAdvancedPane.getModel();
		return this.panel.getModel();
	}
	/**
	 * Gets the GUI
	 */
	public IPublishPluginPanel getPanel() {		
		return this.panel;
	}
	/**
	 * Sets the publication which will be showed by the GUI. Then changes the current visible
	 * panel with the server panel
	 * @param entityModel must be a "publication" object
	 */
	public void setModel(Object entityModel) {
		// cast to publication
		this.publication = (Publication) entityModel;
		//add observer
		publication.getServer().getService(0).addObserver(panel);
		
		//puts the model into the panel
		this.panel.setModel(entityModel);
		
		//update the current server
		currentServer = publication.getServer();
		//update the current service --> if I have only one service for server
		//If you want 2, you must pass an array of services and modify the services panel
		currentService = currentServer.getService(0);
		//update the current resource --> it depends changevalue event (at the begining I get the first rr)
		//if the model contains a remote resource ...
		if (currentService.getRemoteResourcesCount() > 0){
			currentRemoteResource = currentService.getRemoteResource(0);
		}
		//set the panels in the GUI
		setPanels();		
	}
	/**
	 * gets the panel, after updates the model
	 */
	private void setPanels(){
		
		//put server basic
		serverBasicPane = getServerPanel(currentServer);
		//serverBasicPane.setModel(currentServer);
		panel.setServerBasicPanel(serverBasicPane);
		
		//put server advanced
		serverAdvancedPane = getServerAdvancedPanel(currentServer);
		//serverAdvancedPane.setModel(currentServer);
		panel.setServerAdvancedPanel(serverAdvancedPane);
		
		//put service basic
		serviceBasicPane = getServicePanel(currentService);
		//serviceBasicPane.setModel(serviceAdvancedPane);
		panel.setServiceBasicPanel(serviceBasicPane);
		
		//put service advanced
		serviceAdvancedPane = getServiceAdvancedPanel(currentService);
		panel.setServiceAdvancedPanel(serviceAdvancedPane);
		
		//put remote basic
		if (currentRemoteResource !=null){
			remoteBasicPane = getRemoteResourcePanel(currentRemoteResource);
			panel.setRemoteResourceBasicPanel(remoteBasicPane);
		}
		
		//put remote advanced
		if (currentRemoteResource !=null){		
			remoteAdvancedPane = getRemoteResourceAdvancedPanel(currentRemoteResource);
			//remoteAdvancedPane.setModel(currentRemoteResource);
			panel.setRemoteResourceAdvancedPanel(remoteAdvancedPane);
		}
	}
	/**
	 * Actions from buttons
	 */
	public void actionPerformed(ActionEvent e) {
		//first of all I update the model
		getModel();
		if (e.getActionCommand().equals(ADD_RESOURCE_EVENT)){
			AddResourceController addResourceCtrl = new AddResourceController();
			addResourceCtrl.setPublication(publication);
			addResourceCtrl.showWindow();
		}
	}
	/**
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent e) {
		Object source = null;
		if (e.getNewLeadSelectionPath() == null){
			source = e.getOldLeadSelectionPath().getLastPathComponent();
		}else{
			source = e.getNewLeadSelectionPath().getLastPathComponent();
		}
		//update the model
		getModel();
		currentRemoteResource = (RemoteResource)source;		
		//update gui
		setPanels();
	}
	/**
	 * Change tab method
	 * @param e
	 */
	public void stateChanged(ChangeEvent e) {
		getModel();		
		int tab = panel.getSelectedTab();
		if (tab==0){
			panel.setEnabledPrevButton(false);
		}else{ 
			panel.setEnabledPrevButton(true);
		}
		if (tab==2){
			panel.setEnabledNextButton(false);
			//if there isn't resource the addresource use case is called
			if (publication.getServer().getService(0).getRemoteResourcesCount() == 0){
				AddResourceController addResourceCtrl = new AddResourceController();
				addResourceCtrl.setPublication(publication);
				addResourceCtrl.showWindow();
				panel.setSelectedTab(2);				
			}
		}else{
			panel.setEnabledNextButton(true);			
		}
	}

}
