/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.gui.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;

import org.gvsig.publish.gui.publish.IPublishPluginPanel;
import org.gvsig.publish.serversmodel.Publication;
import org.gvsig.publish.serversmodel.Service;

import com.iver.andami.PluginServices;
/**
 * Wizard for showing server properties. Useful when the server has only one service.  
 * 
 * @author jvhigon
 *
 */
public class PublishWizardPanel extends JPanel implements IPublishPluginPanel, ActionListener, Observer{
	/**
	 * Generated versionUID
	 */
	private static final long serialVersionUID = 8029981939846754399L;
	//Associations
	private Publication publication;

	//events
	private final String ISADVANCED_EVENT="isadvanced_event";
	private final String NEXT_PANEL_EVENT="next_panel_event";
	private final String PREV_PANEL_EVENT="prev_panel_event";

	//Constructor
	/**
	 * Default constructor, initialize components and labels
	 */
	public PublishWizardPanel() {		
		initComponents();	
		remoteSplitPane.setDividerLocation(0.5);		
		initLabels();
		initIcons();
	}
	/**
	 * Initialize icons from the classpath
	 */
	private void initIcons() {

		mainTabPanel.setIconAt(0, new javax.swing.ImageIcon(getClass().getClassLoader().getResource("images/publish-server.png")));
		mainTabPanel.setIconAt(1, new javax.swing.ImageIcon(getClass().getClassLoader().getResource("images/publish-service.png")));
		mainTabPanel.setIconAt(2, new javax.swing.ImageIcon(getClass().getClassLoader().getResource("images/publish-rr.png")));
		removeButton.setIcon( new javax.swing.ImageIcon(getClass().getClassLoader().getResource("images/publish-remove.png")));
		addButton.setIcon( new javax.swing.ImageIcon(getClass().getClassLoader().getResource("images/publish-add.png")));
	}
	/**
	 * Initialize gui labels for multilanguage 
	 */
	private void initLabels() {
		addButton.setText(PluginServices.getText(this, "publish_add"));
		removeButton.setText(PluginServices.getText(this, "publish_remove"));
		showAdvanced.setText(PluginServices.getText(this,"publish_show_advanced"));
		prevButton.setText(PluginServices.getText(this,"publish_prev"));
		nextButton.setText(PluginServices.getText(this, "publish_next"));
		mainTabPanel.setTitleAt(0, PluginServices.getText(this, "publish_server"));
		mainTabPanel.setTitleAt(1, PluginServices.getText(this, "publish_service"));
		mainTabPanel.setTitleAt(2, PluginServices.getText(this, "publish_resources"));
	}
	private void initComponents() {

		mainTabPanel = new javax.swing.JTabbedPane();
		serverTab = new javax.swing.JScrollPane();
		serverPane = new javax.swing.JPanel();
		serverBasicPane = new javax.swing.JPanel();
		serverAdvancedPane = new javax.swing.JPanel();
		serviceTab = new javax.swing.JScrollPane();
		servicePane = new javax.swing.JPanel();
		serviceBasicPane = new javax.swing.JPanel();
		serviceAdvancedPane = new javax.swing.JPanel();
		remoteTab = new javax.swing.JPanel();
		remoteSplitPane = new javax.swing.JSplitPane();
		rightScrollPane = new javax.swing.JScrollPane();
		remotePane = new javax.swing.JPanel();
		remoteBasicPane = new javax.swing.JPanel();
		remoteAdvancedPane = new javax.swing.JPanel();
		leftScrollPane = new javax.swing.JScrollPane();
		leftPane = new javax.swing.JPanel();
		remoteTreePane = new javax.swing.JScrollPane();
		remoteTree = new javax.swing.JTree();
		actionsPane = new javax.swing.JPanel();
		addButton = new org.gvsig.gui.beans.swing.JButton();
		removeButton = new org.gvsig.gui.beans.swing.JButton();
		buttonsPanel = new javax.swing.JPanel();
		showAdvanced = new javax.swing.JCheckBox();
		prevButton = new org.gvsig.gui.beans.swing.JButton();
		nextButton = new org.gvsig.gui.beans.swing.JButton();

		setBorder(javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2));
		setLayout(new java.awt.BorderLayout());

		serverPane.setLayout(new java.awt.BorderLayout());
		serverPane.add(serverBasicPane, java.awt.BorderLayout.CENTER);
		serverPane.add(serverAdvancedPane, java.awt.BorderLayout.SOUTH);

		serverTab.setViewportView(serverPane);

		mainTabPanel.addTab("server_tab", serverTab);

		servicePane.setLayout(new java.awt.BorderLayout());
		servicePane.add(serviceBasicPane, java.awt.BorderLayout.CENTER);
		servicePane.add(serviceAdvancedPane, java.awt.BorderLayout.SOUTH);

		serviceTab.setViewportView(servicePane);

		mainTabPanel.addTab("service_tab", serviceTab);

		remoteTab.setLayout(new java.awt.BorderLayout());

		remoteSplitPane.setDividerLocation(150);

		remotePane.setLayout(new java.awt.BorderLayout());
		remotePane.add(remoteBasicPane, java.awt.BorderLayout.CENTER);
		remotePane.add(remoteAdvancedPane, java.awt.BorderLayout.SOUTH);

		rightScrollPane.setViewportView(remotePane);

		remoteSplitPane.setRightComponent(rightScrollPane);

		leftPane.setLayout(new java.awt.BorderLayout());

		remoteTree.setMinimumSize(new java.awt.Dimension(50, 0));
		remoteTree.setRootVisible(false);
		remoteTreePane.setViewportView(remoteTree);

		leftPane.add(remoteTreePane, java.awt.BorderLayout.CENTER);

		actionsPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

		addButton.setText("add");
		actionsPane.add(addButton);

		removeButton.setText("remove");
		actionsPane.add(removeButton);

		leftPane.add(actionsPane, java.awt.BorderLayout.SOUTH);

		leftScrollPane.setViewportView(leftPane);

		remoteSplitPane.setLeftComponent(leftScrollPane);

		remoteTab.add(remoteSplitPane, java.awt.BorderLayout.CENTER);

		mainTabPanel.addTab("remote_tab", remoteTab);

		add(mainTabPanel, java.awt.BorderLayout.CENTER);

		buttonsPanel.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
		buttonsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

		showAdvanced.setText("show_advanced");
		buttonsPanel.add(showAdvanced);

		prevButton.setText("prev");
		prevButton.setEnabled(false);
		buttonsPanel.add(prevButton);

		nextButton.setText("next");
		buttonsPanel.add(nextButton);

		add(buttonsPanel, java.awt.BorderLayout.SOUTH);
	}// </editor-fold>
	// Variables declaration - do not modify
	private javax.swing.JPanel actionsPane;
	public org.gvsig.gui.beans.swing.JButton addButton;
	private javax.swing.JPanel buttonsPanel;
	private javax.swing.JPanel leftPane;
	private javax.swing.JScrollPane leftScrollPane;
	private javax.swing.JTabbedPane mainTabPanel;
	private org.gvsig.gui.beans.swing.JButton nextButton;
	private org.gvsig.gui.beans.swing.JButton prevButton;
	private javax.swing.JPanel remoteAdvancedPane;
	private javax.swing.JPanel remoteBasicPane;
	private javax.swing.JPanel remotePane;
	private javax.swing.JSplitPane remoteSplitPane;
	private javax.swing.JPanel remoteTab;
	public javax.swing.JTree remoteTree;
	private javax.swing.JScrollPane remoteTreePane;
	public org.gvsig.gui.beans.swing.JButton removeButton;
	private javax.swing.JScrollPane rightScrollPane;
	private javax.swing.JPanel serverAdvancedPane;
	private javax.swing.JPanel serverBasicPane;
	private javax.swing.JPanel serverPane;
	private javax.swing.JScrollPane serverTab;
	private javax.swing.JPanel serviceAdvancedPane;
	private javax.swing.JPanel serviceBasicPane;
	private javax.swing.JPanel servicePane;
	private javax.swing.JScrollPane serviceTab;
	private javax.swing.JCheckBox showAdvanced;
	// End of variables declaration

	/**
	 * Gets the publication, updates the model from the GUI
	 */
	public Object getModel() {

		return publication;
	}
	/**
	 * sets listeners for all components
	 */
	public void setListener(EventListener listener) {
		//listener for tabs		
		mainTabPanel.addChangeListener((ChangeListener)listener);

		//listener for buttons
		addButton.addActionListener((ActionListener)listener);
		addButton.setActionCommand(PublishWizardController.ADD_RESOURCE_EVENT);

		nextButton.addActionListener(this);
		nextButton.setActionCommand(NEXT_PANEL_EVENT);
		prevButton.addActionListener(this);
		prevButton.setActionCommand(PREV_PANEL_EVENT);

		//listener for checkbox
		this.showAdvanced.addActionListener(this);
		this.showAdvanced.setActionCommand(ISADVANCED_EVENT);

		//listener for the tree
		remoteTree.addTreeSelectionListener((TreeSelectionListener) listener);

	}
	/**
	 * TODO: I am not sure if this is the best location to do that
	 * sets the model in each pluginpanel
	 */
	public void setModel(Object entityModel) {
		this.publication = (Publication)entityModel;
		//initializes the tree
		Service service = publication.getServer().getService(0);
		TreeModel treemodel = new ServiceTreeModel(service);		
		this.remoteTree.setModel(treemodel);	
		expandAll(remoteTree);
	}

	/**
	 * If you set true, you will be able to see the advanced panels
	 * @param isAdvanced
	 */
	public void setAdvanced(boolean isAdvanced) {		
		serverAdvancedPane.setVisible(isAdvanced);
		serviceAdvancedPane.setVisible(isAdvanced);		
		remoteAdvancedPane.setVisible(isAdvanced);		
	}
	/**
	 * sets the server basic panel
	 * @param serverPanel
	 */
	public void setServerBasicPanel(IPublishPluginPanel pluginPanel) {		
		serverPane.remove(serverBasicPane);
		this.serverBasicPane = (JPanel)pluginPanel;

		serverPane.add(serverBasicPane, java.awt.BorderLayout.NORTH);
	}
	public void setServerAdvancedPanel(IPublishPluginPanel pluginPanel) {		
		serverPane.remove(serverAdvancedPane);
		this.serverAdvancedPane = (JPanel)pluginPanel;
		serverPane.add(serverAdvancedPane, java.awt.BorderLayout.CENTER);
		setAdvanced(showAdvanced.isSelected());
	}
	public void setServiceBasicPanel(IPublishPluginPanel pluginPanel) {
		servicePane.remove(serviceBasicPane);
		this.serviceBasicPane=(JPanel)pluginPanel;
		servicePane.add(serviceBasicPane, java.awt.BorderLayout.NORTH);        
	}
	public void setServiceAdvancedPanel(IPublishPluginPanel pluginPanel) {
		if (pluginPanel != null){
			servicePane.remove(serviceAdvancedPane);
			this.serviceAdvancedPane = (JPanel)pluginPanel;		
			servicePane.add(serviceAdvancedPane, java.awt.BorderLayout.CENTER);
			setAdvanced(showAdvanced.isSelected());
		}else{
			servicePane.remove(serviceAdvancedPane);
		}
	}
	public void setRemoteResourceBasicPanel(IPublishPluginPanel pluginPanel) {
		remotePane.remove(remoteBasicPane);
		this.remoteBasicPane = (JPanel)pluginPanel;
		remotePane.add(remoteBasicPane, java.awt.BorderLayout.NORTH);
		//repaint container? why not repaint()?
		remotePane.doLayout();
	}
	public void setRemoteResourceAdvancedPanel(IPublishPluginPanel pluginPanel) {
		remotePane.remove(remoteAdvancedPane);
		if (pluginPanel != null){
			remotePane.remove(remoteAdvancedPane);
			this.remoteAdvancedPane = (JPanel)pluginPanel;
			remotePane.add(remoteAdvancedPane, java.awt.BorderLayout.CENTER);
			setAdvanced(showAdvanced.isSelected());
			remotePane.repaint();
		}
	}
	public void actionPerformed(ActionEvent e) {
		//active advanced properties
		if (e.getActionCommand().equals(ISADVANCED_EVENT)){			
			if (showAdvanced.isSelected()){
				setAdvanced(true);
			}else{
				setAdvanced(false);
			}
			return;
		}
		//click next button
		if (e.getActionCommand().equals(NEXT_PANEL_EVENT)){
			int tab = mainTabPanel.getSelectedIndex();
			if (tab < mainTabPanel.getTabCount()-1){
				mainTabPanel.setSelectedIndex(tab+1);
			}
		}
		//click prev button
		if (e.getActionCommand().equals(PREV_PANEL_EVENT)){
			int tab = mainTabPanel.getSelectedIndex();
			if (tab > 0){
				mainTabPanel.setSelectedIndex(tab-1);
			}
		}		
	}
	/**
	 * Enable/disable the previous button
	 * @param enabled
	 */
	public void setEnabledPrevButton(boolean enabled){
		prevButton.setEnabled(enabled);
	}
	/**
	 * Enable/disable the next button
	 * @param enabled
	 */
	public void setEnabledNextButton(boolean enabled){
		nextButton.setEnabled(enabled);
	}
	/**
	 * Select the tab to be focused
	 * @param tab
	 */
	public void setSelectedTab(int tab){
		mainTabPanel.setSelectedIndex(tab);
	}
	/**
	 * 
	 * @return the select tab
	 */
	public int getSelectedTab(){
		return mainTabPanel.getSelectedIndex();
	}

	/**
	 * Expands all tree nodes
	 * 
	 * @param tree
	 */
	public void expandAll(JTree tree) {
		int row = 0;
		while (row < tree.getRowCount()) {
			tree.expandRow(row);
			row++;
		}
	}
	/*
	 * (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable arg0, Object arg1) {
		if (arg0 instanceof Service){
			Service srv = (Service) arg0;						
			TreeModel treemodel = new ServiceTreeModel(srv);
			this.remoteTree.setModel(treemodel);
			expandAll(remoteTree);			
			//TreePath path = new TreePath(treemodel.getRoot());
			//remoteTree.fireTreeExpanded(path);
		}

	}
//	private void test(Service s){
//	String aux = "";
//	for (int i =0; i < s.getRemoteResourcesCount(); i++ ){
//	aux = aux + s.getRemoteResource(i).getId();
//	}
//	JOptionPane.showMessageDialog(this, aux);
//	}

}
