/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


/**
 * This class define a filter in the project information. You can filter layers with 
 * a specific datasource, avoid grouped layers, ...
 * 
 * @author jvhigon
 *
 */
public class FilterProjectInfo {
	private boolean groupedLayers=true;
	private Set datasources= new HashSet();
	/**
	 * Filter grouped layers. All the layers will be at the same level.
	 * @param groupedLayers must be false in order to remove all grouped layers
	 */
	public void setGroupedLayersEnabled(boolean groupedLayers) {
		this.groupedLayers = groupedLayers;
	}
	/**
	 * @return the groupedLayers
	 */
	public boolean isGroupedLayersEnabled() {
		return groupedLayers;
	}
	/**
	 * 
	 * @param datasourceType which will be enable like IDataSourceInfo.POSTGIS_TYPE, ...
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo
	 */
	public void addAllowedDatasource(String datasourceType ){
		datasources.add(datasourceType);
	}
	/**
	 * Check if a datasource is in the white list of datasources
	 * @param datasourceType
	 * @return
	 */
	public boolean isAllowedDatasource(IDataSourceInfo ds){
		if (ds == null){
			return false;
		}else{
			return datasources.contains(ds.getType());
		}
		
	}
	/**
	 * Utility for convert a tree of LayerInfos in a list of them. It delete the grouped layers.
	 * @param layers
	 * @return
	 */
	public ILayerInfo[] toList(ILayerInfo[] layers){
		ArrayList list = new ArrayList();
		for (int i = 0; i < layers.length; i++){
			ILayerInfo[] childs = layers[i].getChilds();
			if (childs == null){
				list.add(layers[i]);
			}else{
				ILayerInfo[] aux2 = toList(childs);
				for (int j=0; j< aux2.length; j++){
					list.add(aux2[j]);
				}
			}
		}
		ILayerInfo[] res = new ILayerInfo[list.size()];
		for (int k = 0; k < list.size(); k++){
			res[k] = (ILayerInfo)list.get(k);
		}
		return res;
	}
	
	public boolean isFiltered(ILayerInfo layer){
		//a group is not filtered
		if(layer.getDataSource() == null){
			return false;
		}else{
			boolean aux = false;
			Iterator i = datasources.iterator();
			while (i.hasNext()){
				String dstype = i.next().toString();
				aux = aux || dstype == layer.getDataSource().getDataType();
			}
			return aux;
		}
	}
	
}
