/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject;

import java.awt.geom.Rectangle2D;

public interface IDataSourceInfo {
	//vectorial
	public final String POINT_DATA= "POINT";
	public final String POLYGON_DATA="POLYGON";
	public final String LINE_DATA="LINE";
	public final String MULTI_DATA="MULTI";
	//raster
	public final String BYTE="BYTE";
	public final String INT16="INT16";
	public final String UINT16="UINT16";
	public final String INT32="INT32";
	public final String UINT32="UINT32";
	public final String FLOAT32="FLOAT32";
	public final String FLOAT64="FLOAT64";
	//datasource type
	public final String POSTGIS_TYPE="POSTGIS";
	public final String SHAPE_TYPE="SHAPE";
	public final String RASTER_TYPE="RASTER"; 
	
	/**
	 * 
	 * @return the data source coordinate reference system (in EPSG format)  
	 */
	abstract String getEPSG();
	/**
	 * Gets the system reference identifier of the layer.
	 * @return integer which identifies the layer's CRS. 
	 * If the layer is a group of layers, it will return -1 
	 */
	abstract int getSRID();
	/**
	 * 
	 * @return data type, for vectorial LINE, POINT, POLYGON, MULTI, ...
	 * <p>
	 * for raster  BYTE, INT16, UINT16, INT32, UINT32, FLOAT32, FLOAT64 or null if I don't know the data type
	 * 
	 */

	abstract String getDataType();
	/**
	 * 
	 * @return bounding box in native coordinates
	 */
	abstract Rectangle2D getNativeBBox();
	/**
	 * 
	 * @return bounding box in WGS84 coordinates
	 */
	abstract Rectangle2D getLatLonBBox();
	/**
	 * @return data source type (SHAPE, POSTGIS, RASTER, ..)
	 * 
	 */
	abstract String getType();
	
	/**
	 * 
	 * @return the namespace prefix of the datasource or null if it haven't got it
	 */
	abstract String getNamespacePrefix();
}
