/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package org.gvsig.publish.infoproject;

import java.awt.geom.Rectangle2D;

import org.gvsig.publish.IPublishPersistence;

/**
 * This class represents the information about a layer you are seeing.
 * This class might be reprojected.
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public interface ILayerInfo extends IPublishPersistence  {	
	/**
	 * 
	 * @return layer's title. It must be unique.
	 */
	abstract String getName();
	/**
	 * 
	 * @return layer's title.
	 */
	abstract String getTitle();
	/**
	 * Gets the system reference identifier of the layer.
	 * @return integer which identifies the layer's CRS. 
	 * If the layer is a group of layers, it will return -1 
	 */
	abstract int getSRID();
	/**
	 * @deprecated use getSRID instead
	 * @return string with the epsg code "EPSG:xxxx"
	 */
	abstract String getEPSGString();
	/**
	 * 
	 * @return minimum scale (layer coordinates) you can see the layer  
	 */
	abstract double getMinscale();
	/**
	 * 
	 * @return maximum scale (layer coordinates) you can see the layer
	 */
	abstract double getMaxscale();
	/**
	 * 
	 * @return geographic bounding box in layer's coordinates
	 */
	abstract Rectangle2D getBBox();	
	/**
	 * 
	 * @return legend information
	 */
	abstract ILegendInfo getLegend();
	/**
	 * 
	 * @return datasource information 
	 */
	abstract IDataSourceInfo getDataSource();
	/**
	 * 
	 * @return information about child layers
	 */
	abstract ILayerInfo[] getChilds();
	/**
	 * Set childs and update the parent of each one.
	 * @param childs array of layerinfo
	 */
	abstract void setChilds(ILayerInfo[] childs);
	/**
	 * 
	 * @return parent layer information. Null if there is not parent (no hierarchy)
	 */
	abstract ILayerInfo getParent();
	/**
	 * 
	 * @param parent information about the parent
	 */
	abstract void setParent(ILayerInfo parent);
	/**
	 * 
	 * @return information about the layer's view
	 */
	abstract IViewInfo getViewInfo();
	/**
	 * 
	 * @return information about the layer labeling. It can be null.
	 */
	abstract ILabelingInfo getLabeling();
	
	/**
	 * 
	 * @return the opacity percentage, 0 is transparent, 100 is opaque
	 */
	abstract int getOpacityPercent();

	/**
	 * 
	 * @param layerName layer's name
	 * @return information about the layer
	 */
	abstract ILayerInfo findLayer(String layerName);
	/**
	 * 
	 * @return a string which identifies the hierarchy of layers. 
	 * If the layer is on the top the hierarchy, the string "/" is returned 
	 */
	abstract String getPathString();
}
