/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/**
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 */
package org.gvsig.publish.infoproject;

import java.awt.geom.Rectangle2D;


/**
 * This class represents the view information to publish
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public interface IViewInfo {
	
	/**
	 * 
	 * @return String name of the view
	 */
	abstract String getName();
	/**
	 * 
	 * @return String creation date of the view
	 */
	abstract String getCreationDate();
	/**
	 * 
	 * @return String comments about the view
	 */
	abstract String getComment();
	/**
	 * 
	 * @return String owner who creates the view
	 */
	abstract String getOwner();
	/**
	 * 
	 * @return String identifies the geographic coordinate system of the view resources.
	 * For example EPSG:23030 
	 */
	abstract String getEPSG();
	/**
	 * 
	 * @return the bounding box of the view
	 */
	abstract Rectangle2D getBBox();	
	/**
	 * 
	 * @return LayerInf[] all the layers in the view 
	 */
	abstract ILayerInfo[] getLayers();
	/**
	 * 
	 * @param layerName
	 * @return the information about a layer with unique name "layerName"
	 */
	abstract ILayerInfo findLayer(String layerName);
	/**
	 * 
	 * @return information about the project (parent)
	 */
	abstract IProjectInfo getProjectInfo();		
}
