/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject;

import java.awt.geom.Rectangle2D;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.infoproject.datasources.DataSourceInfoFactory;
import org.gvsig.publish.infoproject.datasources.IRasterDSInfo;
import org.gvsig.publish.infoproject.legends.LegendInfoFactory;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.rendering.Legend;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.cit.gvsig.project.documents.view.ProjectView;
import com.iver.utiles.XMLEntity;
/**
 * TODO: When the tree changes I'm creating a new LayerInfo
 * Is this correct? 
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class LayerInfo implements ILayerInfo {
	private FLayer layer = null;
	private ProjectView view=null;
	private ILayerInfo parent = null;
	private ILayerInfo[] childs = null;	
	private IViewInfo iview = null;
	
	/**
	 * Constructor
	 * @param flayer from gvSIG
	 * @param view from gvSIG. It contains a set of layers
	 * @param viewInfo information about the view in which this layer is 
	 * @param parent information about the parent layer. If it hasn't parent you can use null
	 * 
	 * 
	 * TODO: I must do an exception if the layer is not a FLyrVect or FLyrRaster 
	 */
	public LayerInfo(FLayer flayer, ProjectView view, IViewInfo viewInfo, LayerInfo parent){
		this.parent = parent;
		this.iview = viewInfo;
		this.layer = flayer;
		this.view=view;
		if (flayer instanceof FLayers){
			createChilds();	
		}
	}
	private void createChilds(){		
		FLayers flayers = (FLayers)layer;
		childs = new ILayerInfo[flayers.getLayersCount()];
		//I start in the second element in order to avoid the flayer itself????
		int j = flayers.getLayersCount() -1;
		for (int i = 0; i< flayers.getLayersCount();i++){
			childs[j] = new LayerInfo(flayers.getLayer(i),view, iview, this);
			j--;
		}
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getDataSource()
	 */
	public IDataSourceInfo getDataSource() {				
		return DataSourceInfoFactory.getInstance(this.layer);		
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getLegend()
	 */
	public ILegendInfo getLegend() {		
		return LegendInfoFactory.getInstance(this.layer);
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getMinscale()
	 */
	public double getMinscale() {
		return layer.getMinScale();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getName()
	 */
	public String getName() {		
		return this.layer.getName();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getTitle()
	 */
	public String getTitle() {
		return this.layer.getName();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getChilds()
	 */
	public ILayerInfo[] getChilds() {
		
		return childs;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.ILayerInfo#setChilds(org.gvsig.publish.infoproject.ILayerInfo[])
	 */
	public void setChilds(ILayerInfo[] childs){
		if (childs !=null){
			this.childs =  childs;
			for (int i=0; i< this.childs.length; i++){
				childs[i].setParent(this);
			}
		}else{
			this.childs = null;
		}
		
	}
	/**
	 * 
	 * @see java.lang.Object#toString()
	 */	
	public String toString(){
		return getName();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getViewInfo()
	 */
	public IViewInfo getViewInfo() {		
		return iview;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getEPSGString()
	 */
	public String getEPSGString() {
		return layer.getProjection().getAbrev();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getBBox()
	 */
	public Rectangle2D getBBox() {
		Rectangle2D extent = null;
		try {
			extent = layer.getFullExtent();
		} catch (DriverException e) {
			PublishLogger.getLog().error("ERROR LayerInfo: I can't get the extent", e);
		} 
		return extent;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getMaxscale()
	 */
	public double getMaxscale() { 
		return layer.getMaxScale();
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.ILayerInfo#getOpacityPercent()
	 */
	public int getOpacityPercent() {

		if (layer instanceof FLyrVect){
			FLyrVect v = (FLyrVect)layer;
			FSymbol s = (FSymbol)v.getLegend().getDefaultSymbol();
			int transparency = 100;
			//if a polygon is not filled getColor() returns null
			if (s.getColor() != null){			
				 transparency= s.getColor().getAlpha();
			}
			double opacity = transparency * 0.39;
			opacity = Math.ceil(opacity);
			return new Double(opacity).intValue();
		}else{
			//I get the transparency from the datasource 
			IDataSourceInfo ds = getDataSource();
			if (ds.getType()== IDataSourceInfo.RASTER_TYPE){
				IRasterDSInfo r = (IRasterDSInfo)ds;
				int transparency =  r.getTransparency();
				double opacity = transparency * 0.39;
				opacity = Math.ceil(opacity);
				opacity = 100 - opacity;
				return new Double(opacity).intValue();
			}else
				return 100;
		}

	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getLabeling()
	 */
	public ILabelingInfo getLabeling() {
		if (layer instanceof FLyrVect){
			FLyrVect v = (FLyrVect)layer;
			Legend leg = v.getLegend();
			if (leg instanceof VectorialLegend) {
				return new LabelingInfo(v); 
			}
		}		
		return null;		
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getParent()
	 */
	public ILayerInfo getParent() {
		return this.parent;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.ILayerInfo#setParent(org.gvsig.publish.infoproject.ILayerInfo)
	 */
	public void setParent(ILayerInfo parent){
		this.parent = parent;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#findLayer(java.lang.String)
	 */
	public ILayerInfo findLayer(String layerName) {
		if (getName().equals(layerName)){
			return this;
		}else{
			//find the layer in its childs
			ILayerInfo[] childs = getChilds();
			if (childs == null){
				return null;
			}
			for (int i = 0; i < childs.length; i++ ){
				ILayerInfo target;
				target = childs[i].findLayer(layerName);
				if (target != null){
					return target;
				}
			}
		}
		return null;
	}

/*
 * (non-Javadoc)
 * @see com.iver.utiles.IPersistance#getXMLEntity()
 */
	public XMLEntity getXMLEntity() {		
		XMLEntity xml=new XMLEntity();
		//put version and className
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		
		//put properties
		xml.putProperty("layername", getName());
		return xml;
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version = xml.getIntProperty("version");		
		if (version!= getVersion()){
			PublishLogger.getLog().error("ERROR: " + getClassName() + ": the version doesn't match!");
			return;
		}	
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getSRID()
	 */
	public int getSRID() {
		int srid=-1;
		if (layer.getProjection() != null){
			String[] components = layer.getProjection().getAbrev().split(":");
			srid = new Integer(components[1]).intValue();
		}
		return srid;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.ILayerInfo#getPathString()
	 */
	public String getPathString() {
		if (parent == null){
			return "/";
		}else{
			return parent.getPathString() +  parent.getTitle() + "/";
		}
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.IPublishPersistence#getVersion()
	 */
	public int getVersion() {		
		return 1;
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getClassName()
	 */
	public String getClassName() {
		return "LayerInfo";
	}


}
