/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject;

import java.util.ArrayList;
/**
 * gvSIG objects filter processor.
 * <p>
 * Pattern used: singleton 
 * 
 * @author jvhigon
 *
 */
public class ProcessFilterProjectInfo {
	private static ProcessFilterProjectInfo instance = null;
	
	public static ProcessFilterProjectInfo processor(){
		if (instance == null){
			instance = new ProcessFilterProjectInfo();
		}
		return instance;
	}
	
	/**
	 * Process a filter in this array of LayerInfo.
	 * 1st filter groups.
	 * 2nd filter data sources allowed. 
	 * @param filter
	 * @param layer
	 * @return the LayerInfo processed 
	 */
	public ILayerInfo[] process(FilterProjectInfo filter, ILayerInfo[] layers){
		if (filter == null){
			return layers;
		}
		ILayerInfo[] res = (ILayerInfo[])layers;
		if (filter.isGroupedLayersEnabled()==false){
			res = filterGroups(layers);
		}				
		res = filterDatasources(res,filter);
		
		return res;
	}
	/**
	 * Convert a tree of LayerInfos in a list of them. Grouped layers are deleted.
	 * @param layers
	 * @return
	 */
	private ILayerInfo[] filterGroups(ILayerInfo[] layers){
		ArrayList list = new ArrayList();
		for (int i = 0; i < layers.length; i++){
			ILayerInfo[] childs = layers[i].getChilds();
			if (childs == null){
				list.add(layers[i]);
			}else{
				ILayerInfo[] aux2 = filterGroups(childs);
				for (int j=0; j< aux2.length; j++){
					list.add(aux2[j]);
				}
			}
		}
		ILayerInfo[] res = new ILayerInfo[list.size()];
		for (int k = 0; k < list.size(); k++){
			res[k] = (ILayerInfo)list.get(k);
		}
		return res;
	}
	/**
	 * Delete the layers which datasource is not allowed. 
	 * @param layers
	 * @param filter
	 * @return
	 */
	private ILayerInfo[] filterDatasources(ILayerInfo[] layers, FilterProjectInfo filter) {
		ArrayList list = new ArrayList();
		for(int i = 0; i < layers.length; i++){
			if (layers[i].getChilds() == null){ //is not a group
				if (filter.isAllowedDatasource(layers[i].getDataSource())){
					list.add(layers[i]);					
				}
			}else{
				ILayerInfo[] aux = filterDatasources(layers[i].getChilds(),filter);
				if (aux.length !=0){
					layers[i].setChilds(aux);
					list.add(layers[i]);
				}								
			}				
		}
		ILayerInfo[] res = new ILayerInfo[list.size()];
		for (int k = 0; k < list.size(); k++){
			res[k] = (ILayerInfo)list.get(k);
		}
		return res;
	}
}
