/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/**
 * This class implements IProjectInfo in order to give information about the gvSIG project
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 */
package org.gvsig.publish.infoproject;

import java.util.ArrayList;
import java.util.Iterator;

import org.gvsig.publish.PublishLogger;

import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.documents.view.ProjectView;
import com.iver.cit.gvsig.project.documents.view.ProjectViewFactory;

/**
 * This class represents a gvSIG project information. 
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class ProjectInfo implements IProjectInfo {
	//associations
	private Project gvProject = null;
	private ViewInfo[] views = null;
	private FilterProjectInfo filter;
	/**
	 * Constructor
	 * 
	 * @param prj gvSIG Project
	 * 
	 */
	public ProjectInfo(Project gvProj){
		this.gvProject = gvProj;	
		ArrayList aux = gvProject.getDocumentsByType(ProjectViewFactory.registerName);
		views = new ViewInfo[aux.size()];
		int i = 0;		
		for (Iterator it = aux.iterator(); it.hasNext(); i++){
			ProjectView v = (ProjectView) it.next();
			views[i] = new ViewInfo(v,this);
		}
	}
	/**
	 * 
	 * @see {@link IProjectInfo#getName()}
	 */
	public String getName() {
		return this.gvProject.getName();
	}
	/**
	 * @return the gvSIG project owner
	 * @see org.gvsig.publish.infoproject.IProjectInfo#getOwner()
	 */
	public String getOwner() {
		return this.gvProject.getOwner();
	}
	/**
	 * Creates an array of view information each time is called.
	 * 
	 * @return an array of gvSIG views information
	 * @see org.gvsig.publish.infoproject.IProjectInfo#getViewsInfo()
	 */
	public IViewInfo[] getViewsInfo() {		
		
		return views;
	}
	/**
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		return getName();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IProjectInfo#findLayer(java.lang.String)
	 */
	public ILayerInfo findLayer(String layerName) {
		IViewInfo[] iview = getViewsInfo();
		for (int i = 0; i < iview.length; i++ ){
			ILayerInfo ilayer = iview[i].findLayer(layerName);
			if (ilayer != null){
				return ilayer;
			}
		}
		return null;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IProjectInfo#setFilter(org.gvsig.publish.infoproject.FilterProjectInfo)
	 */
	public void setFilter(FilterProjectInfo filter) {
		this.filter = filter;		
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IProjectInfo#getFilterProjectInfo()
	 */
	public FilterProjectInfo getFilterProjectInfo() {
		return this.filter;
	}
	/**
	 * Clones object
	 */
	public Object clone(){	
		Object obj=null;
		try{
			obj=super.clone();
		}catch(CloneNotSupportedException ex){
			PublishLogger.getLog().error("ERROR " + getClass().getName() + ":I can't clone projectinfo", ex);
		}
		return obj;
	}
}
