/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/**
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 */
package org.gvsig.publish.infoproject;

import java.awt.geom.Rectangle2D;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.project.documents.view.ProjectView;


/**
 * This class represents the information about a gvSIG view
 * 
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class ViewInfo implements IViewInfo {
	private ProjectView view = null;
	private LayerInfo[] layers = null;
	private ProjectInfo project = null;	
	
	/**
	 * Constructor
	 * 
	 */
	public ViewInfo(ProjectView view, ProjectInfo projectInfo){
		this.project = projectInfo;
		this.view = view;
		FLayers lyrs = this.view.getMapContext().getLayers();		
		this.layers = new LayerInfo[lyrs.getLayersCount()];
		int j = lyrs.getLayersCount() -1 ;
		for (int i=0; i < lyrs.getLayersCount(); i++){
			FLayer lyr = lyrs.getLayer(i);
			LayerInfo lyrinf = new LayerInfo(lyr,view, this, null);			
			this.layers[j] = lyrinf;
			j--;
		}	

	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IViewInfo#getComment()
	 */
	public String getComment() {

		return this.view.getComment();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IViewInfo#getCreationDate()
	 */
	public String getCreationDate() {

		return this.view.getCreationDate();
	}
	/**
	 * @see ViewInf.getName()
	 */
	public String getName() {

		return this.view.getName();
	}
	/**
	 * @see ViewInf.getOwner()
	 */
	public String getOwner() {

		return this.view.getOwner();
	}
	/**
	 * @see ViewInf.getSRS()
	 */
	public String getEPSG() {

		return this.view.getMapContext().getProjection().getAbrev();
	}
	/**
	 * @see ViewInfo.IgetLayers()
	 */
	public ILayerInfo[] getLayers() {		
		return ProcessFilterProjectInfo.processor().process(getProjectInfo().getFilterProjectInfo(), this.layers);
	}
	/**
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		return getName();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoProject.IViewInfo#getLayerInfoByName(java.lang.String)
	 */
	public ILayerInfo findLayer(String layerName) {
		ILayerInfo[] ilayers = getLayers();
		for (int i = 0; i < ilayers.length; i++){
			ILayerInfo target = ilayers[i].findLayer(layerName);
			if (target != null){
				return target;
			}			
		}
		return null;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IViewInfo#getBBox()
	 */
	public Rectangle2D getBBox() {
		Rectangle2D aux = null;
		try {
			aux = this.view.getMapContext().getFullExtent();
		} catch (DriverException e) {
			PluginServices.getLogger().error("ERROR ViewInfo: Can't get Full extent", e);
		}
		return aux;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IViewInfo#getProjectInfo()
	 */
	public IProjectInfo getProjectInfo() {
		return project;
	}	

}
