/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject.datasources;

import org.gvsig.publish.infoproject.IDataSourceInfo;

import com.iver.cit.gvsig.fmap.drivers.jdbc.postgis.PostGisDriver;
import com.iver.cit.gvsig.fmap.drivers.raster.CmsRasterDriver;
import com.iver.cit.gvsig.fmap.drivers.shp.IndexedShpDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;

/**
 * This class has the responsability of create a DataSourceInfo instance from 
 * a Layer
 * TODO: Change the possibility of return a null instance by an Exception
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class DataSourceInfoFactory {
	
	/**
	 * Gets a DataSourceInfo from a gvSIG FLayer
	 * TODO: check if getSource() is null, if layer is null,... when a layer is not loaded correctly we can have this situation
	 * @param layer
	 * @return
	 */
	public static IDataSourceInfo getInstance(FLayer layer){ 
		if (layer instanceof FLyrVect){
			FLyrVect lyrVect = (FLyrVect)layer;			
			if (lyrVect.getSource().getDriver() instanceof IndexedShpDriver){
				ShapefileInfo shp = new ShapefileInfo((VectorialAdapter)lyrVect.getSource());  
				//the shapefile don't known the EPSG and I must put it				
				String epsg = layer.getProjection().getAbrev();
				shp.setEPSG(epsg);
				return shp;
			}
			if (lyrVect.getSource().getDriver() instanceof PostGisDriver){
				return new PostGISTableInfo((VectorialAdapter)lyrVect.getSource());
			}
		}
		if (layer instanceof FLyrRaster){
			FLyrRaster lyrRaster = (FLyrRaster)layer;
			if (lyrRaster.getSource().getDriver() instanceof CmsRasterDriver){
				return new RasterInfo((CmsRasterDriver)lyrRaster.getSource().getDriver());
			}
		}
		return null;
	}
		
	
}
