/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject.datasources;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.infoproject.IDataSourceInfo;

import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.ConnectionJDBC;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.db.utils.ConnectionWithParams;
import com.iver.cit.gvsig.fmap.drivers.db.utils.SingleVectorialDBConnectionManager;
import com.iver.cit.gvsig.fmap.drivers.jdbc.postgis.PostGisDriver;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;

public class PostGISTableInfo implements IDataBaseInfo, IDataSourceInfo {
	private VectorialAdapter adapter = null;
	private PostGisDriver driver = null;
	private ConnectionJDBC connection = null;
	private DBLayerDefinition layerDef = null;
	private ConnectionWithParams cwp = null;
	/**
	 * Constructor
	 * @param vectorialAdapter The adapter from gvSIG
	 */
	public PostGISTableInfo(VectorialAdapter vectorialAdapter) {
		this.adapter = vectorialAdapter;
		initialize();		
	}
	/**
	 * Initiliazes the object.
	 * Gets the driver, the connection, the layer definition, the conection with params ...
	 * TODO: I'm running this code a lot of times ..
	 */
	private void initialize() {
		this.driver = (PostGisDriver) adapter.getDriver();
		this.connection = (ConnectionJDBC) driver.getConnection();
		this.layerDef =driver.getLyrDef();
		cwp = SingleVectorialDBConnectionManager.instance().findConnection(this.connection);
		//TODO: improve performance, I'm getting all the connections for each layer!
//		ConnectionWithParams[] conections = SingleVectorialDBConnectionManager.instance().getAllConnections();
//		for (int i = 0; i < conections.length; i++){
//			IConnection con = conections[i].getConnection();  
//			if ( con instanceof ConnectionJDBC){
//				ConnectionJDBC conjdbc = (ConnectionJDBC)con;
//				if (conjdbc.equals(this.connection)){
//					cwp = conections[i];
//				}
//			}			
//		}
				
	}

	/**
	 * 
	 * @return connection host
	 * TODO: I think is not good idea to parse the URL with splits ... 
	 * Exceptions 
	 */
	public String getHost() {
		if (cwp != null){
			return cwp.getHost();
		}else{
			return null;
		}
//		try {
//			String aux = connection.getURL().split("//")[1];
//			return aux.split(":")[0];			
//		} catch (DBException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		return null;
	}
	/**
	 * @return connection port
	 */
	public String getPort() { 
		if (cwp != null){					
			return cwp.getPort();
		}else{
			return null;
		}
		//return driver.getDefaultPort();
	}
	/**
	 * Gets the user of the connection
	 * @return connection user
	 * TODO: Exceptions ...
	 */
	public String getUser() {
		if (cwp != null){
			return cwp.getUser();
		}else{
			return null;
		}
	}
	/**
	 *  
	 * @return the password or null if I can't get it
	 */
	public String getPassword(){			
		if (cwp != null){
			return cwp.getPw();
		}else{
			return null;
		}
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.IDataSourceInfo#getBBox()
	 */
	public Rectangle2D getNativeBBox() {
		//this.adapter.getFullExtent();
		Rectangle2D aux = null;
		try {
			aux = this.driver.getFullExtent();
		} catch (IOException e) {
			PublishLogger.getLog().error("ERROR PostGISTable",e);
		}
		return aux;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.IDataSourceInfo#getCRS()
	 * TODO: MAybe already exists this implementation, but I couldn't find it
	 */
	public String getEPSG() {
		try {
			String sql = "SELECT srid FROM geometry_columns WHERE f_table_name = '" + getTableName() + "' AND f_geometry_column = '" + getGeometryColumn() + "'";
			Statement st = connection.getConnection().createStatement();
			ResultSet rs =  st.executeQuery(sql);
			if (rs.first()){
				int aux = rs.getInt("srid"); 
				return "EPSG:" + aux;
			}
		} catch (SQLException e) {			
			PublishLogger.getLog().error("ERROR PostGISTableInfo : SQL exception getting the EPSG",e);
		}
		return null;
		//TODO: I can't get the datasource CRS !!		
		//return this.driver.getDestProjection();
		//return "EPSG:23030";
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.IDataSourceInfo#getDataType()
	 * TODO: MAybe already exists this implementation, but I couldn't find it 
	 */
	public String getDataType() {
		String geomtype = null;
		try {
			String sql = "SELECT type FROM geometry_columns WHERE f_table_name = '" + getTableName() + "' AND f_geometry_column = '" + getGeometryColumn() + "'";
			Statement st = connection.getConnection().createStatement();
			ResultSet rs =  st.executeQuery(sql);
			if (rs.first()){
				geomtype = rs.getString("type"); 
			}
		} catch (SQLException e) {			
			PublishLogger.getLog().error("ERROR PostGISTableInfo : SQL exception getting the geometry type",e);
		}
	
		if (geomtype.equals("POINT") || geomtype.equals("MULTIPOINT")){
			return IDataSourceInfo.POINT_DATA;
		}
		if (geomtype.equals("LINESTRING") || geomtype.equals("MULTILINESTRING")){
			return IDataSourceInfo.LINE_DATA;
		}
		if (geomtype.equals("POLYGON") || geomtype.equals("MULTIPOLYGON")){
			return IDataSourceInfo.POLYGON_DATA;
		}		
		return null;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.IDataSourceInfo#getType()
	 */
	public String getType() {
		return IDataSourceInfo.POSTGIS_TYPE;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.datasources.IDataBaseInfo#getTableName()
	 */
	public String getTableName() {		
		return this.driver.getTableName();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.datasources.IDataBaseInfo#getGeometryColumn()
	 */
	public String getGeometryColumn() {		
		return layerDef.getFieldGeometry();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.datasources.IDataBaseInfo#getDatabaseName()
	 */
	public String getDatabaseName() {
		if (cwp != null){
			return cwp.getDb();
		}else{
			return null;
		}
		//return layerDef.getCatalogName();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.datasources.IDataBaseInfo#getGID()
	 */
	public String getGID() {		
		return layerDef.getFieldID();
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getLatLonBBox()
	 */
	public Rectangle2D getLatLonBBox() {
		IProjection proj_source = CRSFactory.getCRS(getEPSG());
		IProjection proj_dest = CRSFactory.getCRS("EPSG:4326");
		ICoordTrans trans = proj_source.getCT(proj_dest);		
		return trans.convert(getNativeBBox());
	}
	/**
	 * 
	 * @return the connection name
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getDatastoreName()
	 */
	public String getConnectionName() {		
		return cwp.getName();
	}
	/**
	 * @return schema name
	 * @see org.gvsig.publish.infoproject.datasources.IDataBaseInfo#getSchemaName()
	 */
	public String getSchemaName() {
		return driver.getLyrDef().getSchema();
//		String aux = cwp.getSchema();
//		if (aux == null){
//			aux = "public";
//		}
//		return aux;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getSRID()
	 */
	public int getSRID() {
		int srid=-1;
		String[] components = getEPSG().split(":");
		srid = new Integer(components[1]).intValue();
		return srid;
	}
	/**
	 * @return the name of the connection as prefix
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getNamespacePrefix()
	 */
	public String getNamespacePrefix() {		
		return getConnectionName() + "_" + getSchemaName();
	}
}
