/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject.datasources;

import java.awt.geom.Rectangle2D;
import java.io.File;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.publish.infoproject.IDataSourceInfo;

import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.raster.CmsRasterDriver;
import com.iver.utiles.FileUtils;


public class RasterInfo implements IRasterDSInfo, IFileInfo, IDataSourceInfo {
	private CmsRasterDriver driver = null;
	private Rectangle2D extent = null;

	public RasterInfo(CmsRasterDriver driver) {
		this.driver = driver;
		this.extent = driver.getFullExtent();
	}

	public String getAbsolutePath() {
		String aux = driver.getFile().getAbsolutePath();		
		return aux;
	}

	public String getFileName() {
		return  driver.getFile().getName();		
	}

	public int getNumBands() {

		return driver.getNumBands();
	}

	public Rectangle2D getNativeBBox() {
		return extent;
	}
	/**
	 * Can be null!!
	 */
	public String getEPSG() {
		IProjection proj = driver.getProjection();
		if (proj !=null){
			return proj.getAbrev();
		}
		return null;
	}
	/**
	 * @see http://java.sun.com/docs/books/tutorial/java/nutsandbolts/datatypes.html
	 * @return BYTE, INT16, UINT16, INT32, UINT32, FLOAT32, FLOAT64 or null if I don't know the data type
	 * TODO: complex types CInt16/CInt32/CFloat32/CFloat64
	 *  
	 */
	public String getDataType() {
		int datatype = driver.getRasterDataType();
		switch(datatype){
		case java.awt.image.DataBuffer.TYPE_BYTE:
			return IDataSourceInfo.BYTE;
		case java.awt.image.DataBuffer.TYPE_USHORT:
			return IDataSourceInfo.UINT16;
		case java.awt.image.DataBuffer.TYPE_SHORT:
			return IDataSourceInfo.INT16;
		case java.awt.image.DataBuffer.TYPE_INT:
			return IDataSourceInfo.INT32;
		case java.awt.image.DataBuffer.TYPE_FLOAT:
			return IDataSourceInfo.FLOAT32;
		case java.awt.image.DataBuffer.TYPE_DOUBLE:
			return IDataSourceInfo.FLOAT64;				
		case java.awt.image.DataBuffer.TYPE_UNDEFINED:
			return null;
		}
		return null;
	}
	/**
	 * @return type of datasource, that is RASTER 
	 */
	public String getType() {
		return IDataSourceInfo.RASTER_TYPE;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.datasources.IRasterDSInfo#getTransparency()
	 */
	public int getTransparency() {		
		return this.driver.getTransparency();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.datasources.IFileInfo#getExtension()
	 */
	public String getExtension() {		
		return FileUtils.getFileExtension(driver.getFile());
	}


	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getLatLonBBox()
	 */
	public Rectangle2D getLatLonBBox() {
		IProjection proj_source = CRSFactory.getCRS(getEPSG());
		IProjection proj_dest = CRSFactory.getCRS("EPSG:4326");
		ICoordTrans trans = proj_source.getCT(proj_dest);		
		return trans.convert(getNativeBBox());
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.datasources.IFileInfo#getFile()
	 */
	public File getFile() {		
		return driver.getFile();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.datasources.IFileInfo#getFilenameWithoutExtension()
	 */
	public String getFilenameWithoutExtension() {
		String[] aux = getFileName().split(".");
		if (aux.length > 0){
			return aux[1];
		}else{
			return null;
		}
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getSRID()
	 */
	public int getSRID() {
		int srid=-1;
		String[] components = getEPSG().split(":");
		srid = new Integer(components[1]).intValue();
		return srid;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getNamespacePrefix()
	 */
	public String getNamespacePrefix() {
		// TODO Auto-generated method stub
		return null;
	}
}
