/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject.datasources;


import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.infoproject.IDataSourceInfo;

import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.shp.IndexedShpDriver;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;
import com.iver.utiles.FileUtils;
/**
 * Class that represents a shapefile information 
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 * @see IFileInfo, IVectorialDSInfo
 *
 */
public class ShapefileInfo implements IFileInfo, IDataSourceInfo {	
	private IndexedShpDriver driver = null;
	private VectorialAdapter adapter;
	private String epsg=null;

	/**
	 * Constructor 
	 * @return
	 */
	public ShapefileInfo(VectorialAdapter adapter) {
		this.adapter = adapter;
		this.driver = (IndexedShpDriver) adapter.getDriver();
	}	
	/**
	 * @return the absolute path to the shapefile
	 */
	public String getAbsolutePath() {				
		String aux = driver.getFile().getAbsolutePath();		
		return aux;
	}
	/**
	 * @return the name of the file, on windows, the backslash is replaced by "/" 
	 */
	public String getFileName() {		
		return driver.getFile().getName();
	}
	/**
	 *  
	 * @return the type of the datasource, SHAPEFILE, POSTGIS_TABLE, ... 
	 * 
	 */
//	public String getDataSourceType() {
//	return IVectorialDSInfo.SHAPEFILE;
//	}
	/**
	 * @return the type of the geometries, POINT, POLYGON, LINE, MULTIGEOM, ...
	 * @throws
	 * TODO: Throw a publish exception  
	 */
//	public String getGeomType(){
//	try {
//	switch (adapter.getShapeType()){
//	case FShape.POINT:
//	return IVectorialDSInfo.POINT;
//	case FShape.POLYGON:
//	return IVectorialDSInfo.POLYGON;
//	case FShape.LINE:
//	return IVectorialDSInfo.LINE;
//	}
//	} catch (DriverIOException e) {
//	// TODO Auto-generated catch block
//	e.printStackTrace();
//	}

//	return null;
//	}
	public Rectangle2D getNativeBBox() {
		
		Rectangle2D bbox = null;
		
		//adapter.getFullExtent()		
		try {
			bbox = driver.getFullExtent();
		} catch (IOException e) {
			PublishLogger.getLog().error("ERROR ShapefileInfo",e);
		}
		return bbox;
	}
	/**
	 * 
	 * @param epsg which has been set by the user
	 */
	public void setEPSG(String epsg){
		this.epsg = epsg;
	}
	/**
	 * I can't know the coordinate system of the shapefile. You must set it before.
	 * TODO: see the projection in the prj file 
	 */
	public String getEPSG() {					
		return epsg;
	}
	public String getDataType() {
		try {
			switch (adapter.getShapeType()){
			case FShape.POINT:
				return IDataSourceInfo.POINT_DATA;
			case FShape.POLYGON:
				return IDataSourceInfo.POLYGON_DATA;
			case FShape.LINE:
				return IDataSourceInfo.LINE_DATA;
			}
		} catch (DriverIOException e) {
			PublishLogger.getLog().error("ERROR ShapefileInfo",e);
		}
		return null;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getType()
	 */
	public String getType() {
		return SHAPE_TYPE;
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.datasources.IFileInfo#getExtension()
	 */
	public String getExtension() {
		return FileUtils.getFileExtension(driver.getFile());
	}
	/**
	 * 
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getLatLonBBox()
	 */
	public Rectangle2D getLatLonBBox() {
		IProjection proj_source = CRSFactory.getCRS(getEPSG());
		IProjection proj_dest = CRSFactory.getCRS("EPSG:4326");
		ICoordTrans trans = proj_source.getCT(proj_dest);		
		return trans.convert(getNativeBBox());
	}
	
	/*
	 * 
	 * @see org.gvsig.publish.infoproject.datasources.IFileInfo#getFile()
	 */
	public File getFile() {		
		return driver.getFile();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.datasources.IFileInfo#getFilenameWithoutExtension()
	 */
	public String getFilenameWithoutExtension() {
		String aux = getFileName();
		int i = aux.lastIndexOf(".");		
		return aux.substring(0,i);
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getSRID()
	 */
	public int getSRID() {
		int srid=-1;
		String[] components = getEPSG().split(":");
		srid = new Integer(components[1]).intValue();
		return srid;
	}
	/**
	 * @return the name of the directory which contains the file
	 * @see org.gvsig.publish.infoproject.IDataSourceInfo#getNamespacePrefix()
	 */
	public String getNamespacePrefix() {		
		return getFile().getParentFile().getName();
	}
		
}
