/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject.legends;


import java.util.HashMap;

import org.gvsig.publish.infoproject.ISymbolInfo;
import org.gvsig.publish.infoproject.symbols.SymbolInfoFactory;

import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.rendering.VectorialIntervalLegend;

public class VectorialIntervalLegendInfo implements IIntervalLegendInfo {
	private VectorialIntervalLegend legend = null;
	private HashMap expressions = null;
	private HashMap intervals = null;
	private FLyrVect layer = null;
	/**
	 * constructor 
	 * @param legend
	 * @deprecated
	 */
	public VectorialIntervalLegendInfo(VectorialIntervalLegend legend) {
		this.legend = legend;
		initialize();
	}
	/**
	 * constructor
	 * @param fLyrVect
	 */
	public VectorialIntervalLegendInfo(FLyrVect fLyrVect) {
		this.layer = fLyrVect;
		this.legend = (VectorialIntervalLegend)fLyrVect.getLegend();
		initialize();
	}
	private void initialize(){
		expressions = new HashMap();
		intervals = new HashMap();
		String[] sIntervals = getIntervals();
		String[] sExpressions = new String[sIntervals.length];
		ISymbol[] symbols = legend.getSymbols();
		for (int i=0 ; i<sIntervals.length ; i++){
			sExpressions[i] = createExpression(sIntervals[i]);
			ISymbolInfo symbol = SymbolInfoFactory.getInstance(symbols[i]);
			intervals.put(sIntervals[i], symbol);
			expressions.put(sExpressions[i], symbol);
		}		
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.legends.IIntervalLegendInfo#getFilterItem()
	 */
	public String getFilterItem() {
		return legend.getFieldName();
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.legends.IIntervalLegendInfo#getExpressions()
	 */
	public String[] getExpressions(){
		String[] sIntervals = getIntervals();
		String[] sExpressions = new String[sIntervals.length];
		for (int i=0 ; i<sIntervals.length ; i++){
			sExpressions[i] = createExpression(sIntervals[i]);
		}
		return sExpressions;
	}
	
	/**
	 * Create an expression from an interval
	 * @param interval
	 * An interval (MIN_VALUE - MAXVALUE)
	 * @return
	 * An expression
	 * (FIELD > MIN_VALUE AND FIELD < MAX_VALUE)
	 */
	private String createExpression(String interval){
		String[] intervals = interval.split("-");
		StringBuffer expression = new StringBuffer();
		expression.append("[" + getFilterItem() + "]" + " >= " + formatNumber(intervals[0].trim()));
		expression.append(" AND ");
		expression.append("[" + getFilterItem() + "]" + " <= " + formatNumber(intervals[1].trim()));
		
		return expression.toString();
	}
	
	private String formatNumber(String number){
		number = number.replace(".","");
		return number.replace(",", ".");
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.legends.IIntervalLegendInfo#getIntervals()
	 */
	public String[] getIntervals() {
		return legend.getDescriptions();
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.legends.IIntervalLegendInfo#getSymbolByExpression(java.lang.String)
	 */
	public ISymbolInfo getSymbolByExpression(String expression) {
		if (expressions.containsKey(expression)){
			return (ISymbolInfo)expressions.get(expression);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.legends.IIntervalLegendInfo#getSymbolByInterval(java.lang.String)
	 */
	public ISymbolInfo getSymbolByInterval(String interval) {
		if (intervals.containsKey(interval)){
			return (ISymbolInfo)intervals.get(interval);
		}
		return null;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoproject.ILegendInfo#getSLDString()
	 */
	public String getSLDString() {
		if (layer != null){
			return legend.getSLDString(layer.getName());
		}else{
			return legend.getSLDString("default");
		}
	}

}
