/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.infoproject.symbols;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.net.URI;

import org.gvsig.publish.infoproject.ISymbolInfo;

import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;

public class DefaultSymbolInfo implements ISymbolInfo, IFillSymbolInfo, ILineSymbolInfo, IMarkerSymbolInfo, ITextSymbolInfo{
	private FSymbol symbol = null;
	private Color color = null;
	private double outlineWidth;
	/**
	 * Constructor
	 */
	public DefaultSymbolInfo(Color outlinecolor, double outlinewidth){
		this.color = outlinecolor;
		this.outlineWidth = outlinewidth;
	}
	/**
	 * Constructor from FSymbol
	 * @param symbol
	 */
	public DefaultSymbolInfo(FSymbol symbol) {
		this.symbol = symbol;
		//opacity
		//symbol.getColor().getAlpha();		

	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.ISymbolInfo#getColor()
	 */
	public Color getColor() {
		//is a outline
		if (symbol == null){
			return color;
		}
		//another case
		color = symbol.getColor();
		if (color == null){
			return new Color(255,255,255);
		}
//		int style = symbol.getStyle(); 
//		if (style == FStyle2D.SYMBOL_STYLE_MARKER_TRUETYPE){
//		return symbol.getFontColor();
//		}

		return color;		
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.ISymbolInfo#getTranparency()
	 */
	public int getTranparency() {		
		return symbol.getColor().getAlpha();
	}
	/*
	 * Only points can be in meters!!
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.ISymbolInfo#getUnits()
	 */
	public String getUnits() {
		int type = symbol.getSymbolType();		
		if(type == FShape.POINT ){
			if (!symbol.isSizeInPixels()){
				return ISymbolInfo.METERS;
			}
		}
		return ISymbolInfo.PIXELS;					
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.IFillSymbolInfo#getOutline()
	 */
	public ILineSymbolInfo getOutline() {
		Color aux = symbol.getOutlineColor();
		BasicStroke aux2 = (BasicStroke)symbol.getStroke();
		return new DefaultSymbolInfo(aux, aux2.getLineWidth());
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.ILineSymbolInfo#getLineWitdth()
	 */
	public double getLineWitdth() {		
		if (symbol==null){
			return outlineWidth;
		}
		return ((BasicStroke) symbol.getStroke()).getLineWidth();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.IMarkerSymbolInfo#getSize()
	 */
	public int getSize() {

		return symbol.getSize();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.ISymbolInfo#getTitle()
	 */
	public String getTitle() {

		return symbol.getDescription();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.IMarkerSymbolInfo#getWellKnowSymbolName()
	 */
	public String getWellKnowMarkerName() {
		int style = symbol.getStyle();
		switch (style){
		case FStyle2D.SYMBOL_STYLE_MARKER_CIRCLE:			
			return IMarkerSymbolInfo.WELLKNOWMARKER_CIRCLE;
		case FStyle2D.SYMBOL_STYLE_MARKER_CROSS:
			return IMarkerSymbolInfo.WELLKNOWMARKER_CROSS;
		case FStyle2D.SYMBOL_STYLE_MARKER_SQUARE:
			return IMarkerSymbolInfo.WELLKNOWMARKER_SQUARE;
		case FStyle2D.SYMBOL_STYLE_MARKER_TRIANGLE:
			return IMarkerSymbolInfo.WELLKNOWMARKER_TRIANGLE;		
		default:
			return "UNKNOW";
		}
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.IMarkerSymbolInfo#getImageIconPath()
	 */
	public File getImageIconFile() {
		URI uri = symbol.getIconURI();		

		if (uri != null){
			File f = new File (uri);
			return f;
		}else{
			return null;
		}
		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.ITextSymbolInfo#getFont()
	 */
	public Font getFont() {		
		return symbol.getFont();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.ITextSymbolInfo#getFontSize()
	 */

	public double getFontSize() {
		return symbol.getFontSize();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.infoProject.symbols.ITextSymbolInfo#getFontColor()
	 */
	public Color getFontColor() {
		return symbol.getFontColor();
	}
	/**
	 * @return string which identifies a well know hatch into a polygon. If the polygon is solid the functions returns null
	 *   
	 */
	public String getWellKnowFillName() {
		int style = symbol.getStyle();
		if (style == FStyle2D.SYMBOL_STYLE_FILL_CROSS)
			return IFillSymbolInfo.WELLKNOWFILL_CROSS;
		if (style == FStyle2D.SYMBOL_STYLE_FILL_CROSS_DIAGONAL)
			return IFillSymbolInfo.WELLKNOWFILL_CROSSDIAGONAL;
		if (style == FStyle2D.SYMBOL_STYLE_FILL_DOWNWARD_DIAGONAL)
			return IFillSymbolInfo.WELLKNOWFILL_DOWNWARDDIAGONAL;
		if (style == FStyle2D.SYMBOL_STYLE_FILL_HORIZONTAL)
			return IFillSymbolInfo.WELLKNOWFILL_HORIZONTAL;
		if (style == FStyle2D.SYMBOL_STYLE_FILL_UPWARD_DIAGONAL)
			return IFillSymbolInfo.WELLKNOWFILL_UPWARDDIAGONAL;
		if (style == FStyle2D.SYMBOL_STYLE_FILL_VERTICAL)
			return IFillSymbolInfo.WELLKNOWFILL_VERTICAL;
		return null;
//		if (style == FStyle2D.SYMBOL_STYLE_FILL_LIGHTGRAYFILL)
//		return null;
//		if (style == FStyle2D.SYMBOL_STYLE_FILL_SOLID)
//		return null;
//		if (style == FStyle2D.SYMBOL_STYLE_FILL_TRANSPARENT)
//		return null;
//		if (style == FStyle2D.SYMBOL_STYLE_FILL_DARKGRAYFILL)
//		return null;
//		if (style == FStyle2D.SYMBOL_STYLE_FILL_GRAYFILL)
//		return null;
		
	}
	public String getWellKnowLineName() {
		BasicStroke s = (BasicStroke)symbol.getStroke();
		float[] dasharray = s.getDashArray();
		if (dasharray == null){
			return null;
		}
		if (dasharray.length == 1){
//			if (dasharray[0] == 1.0)
//				return ILineSymbolInfo.WELLKNOWLINE_SIMPLE;
			if (dasharray[0] == 3.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_3;
			if (dasharray[0] == 5.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_5;
			if (dasharray[0] == 7.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_7;
			if (dasharray[0] == 9.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_9;
		}else{
			if (dasharray[0] == 5.0 && dasharray[1] == 1.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_5_1;
			if (dasharray[0] == 7.0 && dasharray[1] == 12.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_7_12;
			if (dasharray[0] == 9.0 && dasharray[1] == 2.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_9_2;
			if (dasharray[0] == 15.0 && dasharray[1] == 6.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_15_6;
			if (dasharray[0] == 20.0 && dasharray[1] == 3.0)
				return ILineSymbolInfo.WELLKNOWLINE_DASH_20_3;
		}								
		return null;	
	}
}
