/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.ogcmetadata;

import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;

import com.iver.utiles.XMLEntity;

/**
 * This class represents any OGC entity metadata 
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class OGCCommonMetadata implements IPublishPersistence, IOGCCommonMetadata{
	
	private String name=null;
	private String title = null;
	private String description = null;
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.IPublishPersistence#getVersion()
	 */
	public int getVersion(){
		return 1;
	}

	/* (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.ICommonOGCMetadata#getName()
	 */
	public String getName(){
		return name;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.ICommonOGCMetadata#setName(java.lang.String)
	 */
	public void setName(String name){
		this.name = name;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.ICommonOGCMetadata#getTitle()
	 */
	public String getTitle(){
		return title;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.ICommonOGCMetadata#setTitle(java.lang.String)
	 */
	public void setTitle(String title){
		this.title = title;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.ICommonOGCMetadata#getAbstract()
	 */
	public String getAbstract(){
		return description;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.ICommonOGCMetadata#setAbstract(java.lang.String)
	 */
	public void setAbstract(String description){
		this.description = description;
	}
/*
 * (non-Javadoc)
 * @see com.iver.utiles.IPersistance#setXMLEntity(com.iver.utiles.XMLEntity)
 */	
	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version = xml.getIntProperty("version");		
		if (version!= getVersion()){
			PublishLogger.getLog().error("ERROR: " + getClassName() + ": the version doesn't match!");
			return;
		}	
		//set properties
		setTitle(xml.getStringProperty("ogctitle"));
		setName(xml.getStringProperty("ogcname"));
		setAbstract(xml.getStringProperty("ogcabstract"));
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() {		
		XMLEntity xml=new XMLEntity();
		//put version and className
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		
		//put properties
		xml.putProperty("ogcname", getName());
		xml.putProperty("ogctitle", getTitle());
		xml.putProperty("ogcabstract", getAbstract());
		return xml;
	}
/*
 * (non-Javadoc)
 * @see com.iver.utiles.IPersistance#getClassName()
 */
	public String getClassName() {
		
		return "OGCCommonMetadata";
	}

}
