/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.ogcmetadata;


import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;

import com.iver.utiles.IPersistance;
import com.iver.utiles.XMLEntity;

/**
 * This class represent a wms 1.1.1 layer metadata
 * @author jvhigon
 *
 */
public class OGCLayerMetadata implements IPublishPersistence, IOGCLayerMetadata{
	private IOGCCommonMetadata icommon = null;
	private IPersistance pcommon=null;
	private boolean isQueryable;
	/**
	 * default constructor
	 */
	public OGCLayerMetadata() {
		icommon = new OGCCommonMetadata();
		pcommon = (IPersistance) icommon;
	}
	/**
	 * 
	 * @return class name and version
	 */
	public int getVersion(){
		return 1;
	}
	
	public boolean isQueryable() {
		return isQueryable;
	}

	public void setQueryable(boolean isQueryable) {
		this.isQueryable = isQueryable;
	}
	
	/**
	 * This function initializes the object from the xml
	 * @param xml 
	 */
	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version= xml.getIntProperty("version");
		if (version != getVersion()){
			PublishLogger.getLog().error("ERROR: " + getClassName() + ": the version doesn't match!");
			return;
		}
		//set properties
		setQueryable(xml.getBooleanProperty("queryable"));	
		//set associations
		pcommon.setXMLEntity(xml.getChild(0));
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getClassName()
	 */
	public String getClassName() {
		return "OGCLayerMetadata";
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml=new XMLEntity();
		//put version and name
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		//put properties
		xml.putProperty("queryable", isQueryable());
		//put associations
		xml.addChild(pcommon.getXMLEntity());		
		return xml;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getAbstract()
	 */
	public String getAbstract() {
		
		return icommon.getAbstract();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getName()
	 */
	public String getName() {
		
		return icommon.getName();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getTitle()
	 */
	public String getTitle() {

		return icommon.getTitle();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setAbstract(java.lang.String)
	 */
	public void setAbstract(String description) {
		
		icommon.setAbstract(description);
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setName(java.lang.String)
	 */
	public void setName(String name) {
		icommon.setName(name);		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setTitle(java.lang.String)
	 */
	public void setTitle(String title) {
		icommon.setTitle(title);		
	}
	
}
