/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibaez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.ogcmetadata;

import java.net.MalformedURLException;
import java.net.URL;

import org.gvsig.publish.IPublishPersistence;
import org.gvsig.publish.PublishLogger;

import com.iver.utiles.XMLEntity;

/**
 * This class represents a service wms 1.1.1 metadata 
 * 
 * @author Jos Vicente Hign (josevicente.higon@iver.es)
 *
 */
public class OGCServiceMetadata implements IPublishPersistence, IOGCServiceMetadata {

	private OGCCommonMetadata common = new OGCCommonMetadata(); 
	private URL onlineresource = null;	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.IPublishPersistence#getVersion()
	 */
	public int getVersion(){
		return 1; 
	}

	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#setXMLEntity(com.iver.utiles.XMLEntity)
	 */	
	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version = xml.getIntProperty("version");
		if (version != getVersion()){
			PublishLogger.getLog().error("ERROR: " + getVersion() + ": the version doesn't match!");
			return;
		}	
		//set properties
//		try{
//			String onlineresource = xml.getStringProperty("onlineresource");		
//			URL url=null;
//			URL aux=null;
//			try {
//				url = new URL(onlineresource);
//				//auxiliary well formed url
//				aux = new URL("http://");
//			} catch (MalformedURLException e1) {
//				PublishLogger.getLog().error("ERROR: ServiceWMS111: malformed url setting the onlineresource from persistence");
//				setOnlineResource(aux);
//			}
//			setOnlineResource(url);
//		}catch(NotExistInXMLEntity e){
//			PublishLogger.getLog().warn("WARNING:" + getClassName() + ": missing onlineresource in persistence", e);
//		}
		String online = xml.getStringProperty("onlineresource");
		if (onlineresource != null){
			URL url = null;
			try {
				url = new URL(online);
				setOnlineResource(url);
			} catch (MalformedURLException e) {
				PublishLogger.getLog().error("WARNING " + getClassName() + " :malformed url setting the onlineresource from persistence");
				setOnlineResource(null);
			}
		}
		//set associations
		common.setXMLEntity(xml.getChild(0));
	}


	public String getClassName() {
		return "OGCServiceMetadata";
	}


	public XMLEntity getXMLEntity() {
		XMLEntity xml=new XMLEntity();
		//put version and name
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		//put properties
		if (getOnlineResource() != null){
			xml.putProperty("onlineresource", getOnlineResource().toString());
		}else{
			xml.putProperty("onlineresource",null);
		}
		//put associations
		xml.addChild(common.getXMLEntity());
		return xml;
	}


	public URL getOnlineResource() {
		return onlineresource;
	}


	public void setOnlineResource(URL url) {
		this.onlineresource = url;
	}


	public String getAbstract() {
		return common.getAbstract();
	}


	public String getName() {
		return common.getName();
	}


	public String getTitle() {
		return common.getTitle();
	}

	public void setAbstract(String description) {
		this.common.setAbstract(description);
	}

	public void setName(String name) {
		this.common.setName(name);
	}

	public void setTitle(String title) {
		this.common.setTitle(title);
	}

}
